#ifndef ObjectState_h_Included             // -*- C++ -*-
#define ObjectState_h_Included   

/// ObjectState -- A class for controlling object states.
/**
 * \ingroup base
 *
 * @author Yvon Halbwachs
 */

class ObjectState
{
public:
  
  /** Creates the initial state */
  ObjectState() : count_(0) {}
  
  /** Copy constructor */
  ObjectState(const ObjectState& other)
    : count_(other.count_) {}
  
  /** State destruction */
  ~ObjectState() {}
  
  /** Indicate that this state has changed */
  inline void touch();
  
  /** Returns true if this state has changed relative to another state */
  inline bool hasChanged(const ObjectState &other) const;
  
  /** Reset other state according to this state */
  inline void clear(ObjectState &other);
  
protected:
  
  /** State counter */
  int count_;
  
};

//----------------------------------------------------------
inline void ObjectState::touch()
//----------------------------------------------------------
{
  count_++;
}

//----------------------------------------------------------
inline bool ObjectState::hasChanged(const ObjectState &other) const
//----------------------------------------------------------
{
  return (other.count_ < count_);
}

//----------------------------------------------------------
inline void ObjectState::clear(ObjectState &other)
//----------------------------------------------------------
{
  other.count_ = count_;
}

#endif // ObjectState_h_Included
