#include <scenegraph/OsgDolfinObject.h>

#include <gui/ApplicationWindow.h>
#include <gui/Context.h>
#include <gui/MeshFunctionsToolbar.h>
#include <gui/ViewerInterface.h>

#include <ui_ApplicationWindow.h>

#include <QFileDialog>
#include <QApplication>
#include <QDockWidget>
#include <QToolBar>
#include <QMessageBox>
#include <QMenuBar>
#include <QMenu>
#include <QInputDialog>
#include <QMdiSubWindow>
#include <QImageWriter>

#include <osgDB/WriteFile>

#include <iostream>

//----------------------------------------------------------
ApplicationWindow::ApplicationWindow(QWidget *parent, Qt::WindowFlags flags)
  : QMainWindow(parent, flags), context_(this)
//----------------------------------------------------------
{
  gui.setupUi(this);

  // Create the toolbar
  meshfunctions_toolbar = new MeshFunctionsToolbar(context_);
  addToolBar(meshfunctions_toolbar);

  connect(gui.m_Exit_,  SIGNAL(activated()), this, SLOT(quit()));
  connect(gui.m_About_, SIGNAL(activated()), this, SLOT(about()));
  connect(gui.m_Undo_,  SIGNAL(activated()), this, SLOT(undo()));

  connect(gui.m_ImportDolfinMesh_,  SIGNAL(activated()), 
	  this, SLOT(importDolfinMesh()));
  
  connect(gui.m_SaveImage_,  SIGNAL(activated()), 
	  this, SLOT(saveImage()));
  
   connect(gui.m_LoadDolfinVertexBoundaryIndicators_,  SIGNAL(activated()), 
 	  this, SLOT(loadDolfinVertexBoundaryIndicators()));

  connect(gui.m_LoadDolfinEdgeBoundaryIndicators_,  SIGNAL(activated()), 
	  this, SLOT(loadDolfinEdgeBoundaryIndicators()));

  connect(gui.m_LoadDolfinFaceBoundaryIndicators_,  SIGNAL(activated()), 
	  this, SLOT(loadDolfinFaceBoundaryIndicators()));

//   connect(gui.m_LoadDolfinBoundaryIndicators_,  SIGNAL(activated()), 
// 	  this, SLOT(loadDolfinBoundaryIndicators()));

  connect(gui.m_SaveDolfinVertexBoundaryIndicators_,  SIGNAL(activated()), 
	  this, SLOT(saveDolfinVertexBoundaryIndicators()));

  connect(gui.m_SaveDolfinEdgeBoundaryIndicators_,  SIGNAL(activated()), 
	  this, SLOT(saveDolfinEdgeBoundaryIndicators()));

  connect(gui.m_SaveDolfinFaceBoundaryIndicators_,  SIGNAL(activated()), 
	  this, SLOT(saveDolfinFaceBoundaryIndicators()));

//   connect(gui.m_SaveDolfinBoundaryIndicators_,  SIGNAL(activated()), 
// 	  this, SLOT(saveDolfinBoundaryIndicators()));

  // Load and Save of all boundary indicators doesn't work yet.
  gui.m_LoadDolfinBoundaryIndicators_->setEnabled(false);
  gui.m_SaveDolfinBoundaryIndicators_->setEnabled(false);

  // Give acces to global undo action.
  gui.m_Undo_->setEnabled(false);
  context_.setUndoAction(gui.m_Undo_);

  // Update application window title with version information
  QString title = QString("%1 v%2.%3")
    .arg(windowTitle())
    .arg(MB_VERSION_MAJOR)
    .arg(MB_VERSION_MINOR);
  setWindowTitle(title);

  setWindowIcon(QIcon(":/resources/dolfin-small.png"));
}

//----------------------------------------------------------
ApplicationWindow::~ApplicationWindow()
//----------------------------------------------------------
{
}
//----------------------------------------------------------
QMdiArea* ApplicationWindow::getMdiArea() 
//----------------------------------------------------------
{
  return gui.w_mdi_area_;
}
//----------------------------------------------------------
const Context *ApplicationWindow::context() const
//----------------------------------------------------------
{
  return &context_;
}

//----------------------------------------------------------
Context *ApplicationWindow::context()
//----------------------------------------------------------
{
  return &context_;
}

// //----------------------------------------------------------
// bool ApplicationWindow::saveScene(const QString &filename)
// //----------------------------------------------------------
// {
//   if (!context_.viewer())
//     return false;

//   Viewer_3D* viewer_3D = dynamic_cast<Viewer_3D*>(context_.viewer());

//   if (viewer_3D != NULL) 
//   {
//     return osgDB::writeNodeFile(*(viewer_3D->getSceneData()),
//                                 filename.toStdString());
//   } else
//   {
//     return false;
//   }
// }

//----------------------------------------------------------
void ApplicationWindow::quit()
//----------------------------------------------------------
{
  // std::cerr << "ApplicationWindow::quit(): Called" << std::endl;
  qApp->quit();
}
//----------------------------------------------------------
void ApplicationWindow::viewAll()
//----------------------------------------------------------
{
  //FIXME: What should happend here?
//   Viewer *viewer = context()->viewer();
//   if (viewer) {

//     // inherited from osgViewer::View
//     viewer->home();

//     //inherited from RenderArea
//     viewer->requestRedraw();
//   }
}

//----------------------------------------------------------
void ApplicationWindow::viewCenter()
//----------------------------------------------------------
{

  ViewerInterface *viewer = context()->viewer();
  viewer->viewCenter();
  
//   OsgTransformObject *osg_obj = viewer->getOsgObj();
//   if (viewer && osg_obj) {
//     viewer->setTrackballCenter(osg_obj);
//     viewer->requestRedraw();
//   }
}
//----------------------------------------------------------
void ApplicationWindow::about()
//----------------------------------------------------------
{
  QMessageBox::aboutQt(this, "About");
}

//----------------------------------------------------------
void ApplicationWindow::importDolfinMesh()
//----------------------------------------------------------
{
  QString filename = 
    QFileDialog::getOpenFileName(this, 
				 "Open/Load project file",
				 QString::null, 
				 "Project files (*.xml *.xml.gz)");

  if(filename.isEmpty())
    return;


  context_.importDolfinMesh(filename);
  

  viewAll();
}

//----------------------------------------------------------
void ApplicationWindow::loadDolfinVertexBoundaryIndicators()
//----------------------------------------------------------
{

  QString filename = 
    QFileDialog::getOpenFileName(this,
				 "Open/Load boundary vertex indicator file",
				 QString::null, 
				 "Boundary indicator files (*.xml *.xml.gz)");
  
  if(!filename.isEmpty())
  {
    currentViewer()->get_mesh().loadVertexBoundaryIndicators(filename);
    currentViewer()->updateBoundaryIndicatorColors();
  }
}

//----------------------------------------------------------
void ApplicationWindow::loadDolfinEdgeBoundaryIndicators()
//----------------------------------------------------------
{
  QString filename = 
    QFileDialog::getOpenFileName(this,
				 "Open/Load boundary edge indicator file",
				 QString::null, 
				 "Boundary indicator files (*.xml *.xml.gz)");
  
  if(!filename.isEmpty())
  {
    currentViewer()->get_mesh().loadEdgeBoundaryIndicators(filename);
    currentViewer()->updateBoundaryIndicatorColors();
  }
}

//----------------------------------------------------------
void ApplicationWindow::loadDolfinFaceBoundaryIndicators()
//----------------------------------------------------------
{
  QString filename = 
    QFileDialog::getOpenFileName(this,
				 "Open/Load boundary face indicator file",
				 QString::null, 
				 "Boundary indicator files (*.xml *.xml.gz)");
  
  if(!filename.isEmpty())
  {
    currentViewer()->get_mesh().loadFaceBoundaryIndicators(filename);
    currentViewer()->updateBoundaryIndicatorColors();
  }
}

// //----------------------------------------------------------
// void ApplicationWindow::loadDolfinBoundaryIndicators()
// //----------------------------------------------------------
// {
//   QString filename = 
//     QFileDialog::getOpenFileName(this,
// 				 "Open/Load boundary indicator file",
// 				 QString::null, 
// 				 "Boundary indicator files (*.xml *.xml.gz)");
  
//   if(!filename.isEmpty())
//     currentViewer()->get_mesh().loadBoundaryIndicators(filename);
  
//   context_.requestRedraw();
// }
//----------------------------------------------------------
void ApplicationWindow::saveDolfinVertexBoundaryIndicators()
//----------------------------------------------------------
{
  QString filename = 
    QFileDialog::getSaveFileName(this, 
				 "Save/Write vertex boundary indicator file", 
				 QString::null,
				 "Boundary indicator files (*.xml)");    
  if(!filename.isEmpty()) {
    if(!filename.endsWith(".xml"))
      filename.append(".xml");

    currentViewer()->get_mesh().saveVertexBoundaryIndicators(filename);
  }
}

//----------------------------------------------------------
void ApplicationWindow::saveDolfinEdgeBoundaryIndicators()
//----------------------------------------------------------
{

  if (currentViewer())
  {
    
    QString filename = 
      QFileDialog::getSaveFileName(this, 
                                   "Save/Write edge boundary indicator file", 
                                   QString::null,
                                   "Boundary indicator files (*.xml)");    
    if(!filename.isEmpty()) {
      if(!filename.endsWith(".xml"))
        filename.append(".xml");
      
      currentViewer()->get_mesh().saveEdgeBoundaryIndicators(filename);
    }
  }
}

//----------------------------------------------------------
void ApplicationWindow::saveDolfinFaceBoundaryIndicators()
//----------------------------------------------------------
{

 
  QString filename = 
    QFileDialog::getSaveFileName(this, 
				 "Save/Write face boundary indicator file", 
				 QString::null,
				 "Boundary indicator files (*.xml)");    
  if(!filename.isEmpty()) {
    if(!filename.endsWith(".xml"))
      filename.append(".xml");
    
    currentViewer()->get_mesh().saveFaceBoundaryIndicators(filename);
  }
}

// //----------------------------------------------------------
// void ApplicationWindow::saveDolfinBoundaryIndicators()
// //----------------------------------------------------------
// {
  
//   if (currentViewer())
//   {
 
//     QString filename = 
//       QFileDialog::getSaveFileName(this, 
//                                    "Save/Write boundary indicator file", 
//                                    QString::null,
//                                    "Boundary indicator files (*.xml)");    
//     if(!filename.isEmpty()) {
//       if(!filename.endsWith(".xml"))
//         filename.append(".xml");
    
//       currentViewer()->get_mesh().saveBoundaryIndicators(filename);
//     }
//   }
// }

//----------------------------------------------------------
void ApplicationWindow::undo()
//----------------------------------------------------------
{
//   OsgDolfinObject *osg_dolfin_object
//     = dynamic_cast<OsgDolfinObject*>(context_.viewer()->getOsgObj());
  
//   if (!osg_dolfin_object) // Return if there was no DolfinObject hit
//     return;

//   osg_dolfin_object->undo();
//   gui.m_Undo_->setEnabled(false);

  currentViewer()->undo();


  //context_.requestRedraw();
}

//----------------------------------------------------------
void ApplicationWindow::saveImage()
//----------------------------------------------------------
{

//     // Redraw and grab image
//   Viewer *viewer = context_.viewer();
//   if (!viewer)
//     return;

//   viewer->updateGL();

//   QImage image = viewer->grabFrameBuffer(false);
    
//   if(image.isNull()) {
//     QMessageBox::critical(this, "Save Image", "Not able to grab framme buffer");
//     return;
//   }

  // Build filter list
  QList<QByteArray> list = QImageWriter::supportedImageFormats();
  QList<QByteArray>::Iterator it;

  QMap<QString, QString> filters;
  QString line,format;

  QString filter = "PNG files (*.png);; All files (*.*)";
  filters["PNG files (*.png)"] = "png"; 

  for(it=list.begin() ; it != list.end() ; it++) {
    // PNG is our prefered format, so it has
    // allready been added to filter list
    if(*it == "png") continue;
    
    format = (*it);
    
    // Special case since Qt uses ".jpeg" and we want 
    // to also use the extension ".jpg"
    if(*it == "jpeg")
      format += " *.jpg";

    line = QString("%1 files (*.%2)").arg(QString(*it)).arg(format);
    
    filter += ";;" + line;

    filters[line] = (*it);
  }
  
  // Open File dialog 
  QString selected_filter;
  QString filename = 
    QFileDialog::getSaveFileName(this, "Save Snapshot", QString::null, 
				 filter, &selected_filter);
  
  // Save image
  // let Qt decide the format based on the filename extension
  // TODO: This might result in an error if Qt doesn't 
  // recognize the filename extension.
  //image.save(filename);  
  currentViewer()->saveImage(filename);
}

// //----------------------------------------------------------
// void ApplicationWindow::addViewer(QWidget *viewer) 
// //----------------------------------------------------------
// {
//   getMdiArea()->addSubWindow(viewer);
//   viewer->show();
// }
//----------------------------------------------------------
ToolBoxDock* ApplicationWindow::getToolBoxDock()
//----------------------------------------------------------
{
  return toolbox_dock;
}
//----------------------------------------------------------
ViewerInterface* ApplicationWindow::currentViewer() 
//----------------------------------------------------------
{
  return context_.viewer();
}
