#ifndef Gui_ApplicationWindow_h_Included             // -*- C++ -*-
#define Gui_ApplicationWindow_h_Included


#include <gui/Context.h>
#include <gui/Viewer.h>
#include <util/SimResState.h>
#include <ui_ApplicationWindow.h>

#include <QMainWindow>

class QToolBar;
class QMenu;
class MeshFunctionsToolbar;

/**
 * Main window application base class.
 *
 * \ingroup gui
 *
 * \author Yvon Halbwachs
 * \author Benjamin Kehlet
 * \date 2010-03-29
 */

class ApplicationWindow : public QMainWindow
{
  Q_OBJECT;
  
public:
  
  /** Constructs a main window with the given parent and the specified flags. */
  ApplicationWindow(QWidget *parent = 0, Qt::WindowFlags flags = 0);
  
  /** Destroys the main window. */
  virtual ~ApplicationWindow();
  
  /** Get global context */
  const Context *context() const;
  
  /** Get global context */
  Context* context();

  /** Get the mdi area */
  QMdiArea* getMdiArea();

  /** Get the singleton Transformsettings */
  TransformSettings* getTransformSettings();

  ToolBoxDock* getToolBoxDock();
  
  /** Return pointer to viewer specific menu */
  QMenu* getViewerMenu() { return gui.menu_Viewer; }

  /** Add a new viewer */
  //void addViewer(QWidget *viewer);
  
  // FIXME: This should not be public, but at the moment
  // DolfinEventHandler needs access to this object.
  MeshFunctionsToolbar* meshfunctions_toolbar;                                 

public slots:
  // Is this functionality needed? Is applies to 3D only, and who want to 
  // save their mesh as an osg file?
  /** Save scene in current viewer as an osg file */
  //virtual bool saveScene(const QString &filename);
  
  /** Quits the application. */
  virtual void quit();
    
  /** View all in current viewer */
  virtual void viewAll();
  
  /** Change cetner of view in current viewer */
  virtual void viewCenter();
  
  /** Shows information about Qt *
   *  Can be redefined by derived classes */
  virtual void about();
  
  /** Uncommented */
  void saveImage();

  /** Import a Dolfin mesh */
  void importDolfinMesh();
  
  /** Load Dolfin mesh boundary indicators*/
   void loadDolfinVertexBoundaryIndicators();
   void loadDolfinEdgeBoundaryIndicators();
   void loadDolfinFaceBoundaryIndicators();
//   void loadDolfinBoundaryIndicators();
  
  /** Save Dolfin mesh boundary indicators */
   void saveDolfinVertexBoundaryIndicators();
   void saveDolfinEdgeBoundaryIndicators();
   void saveDolfinFaceBoundaryIndicators();
//   void saveDolfinBoundaryIndicators();
  
  /** Undo last edit */
  void undo();

  ViewerInterface* currentViewer();
  
private:

  /** Global context */
  Context context_;
  
  /** Current opened file (used for save) */
  QString curr_filename_;

  /** The gui (generated from QT designer) */
  Ui::ApplicationWindowBase gui;

  ToolBoxDock* toolbox_dock;
  //  TransformSettings* transform_settings;
  
};

#endif // Gui_ApplicationWindow_h_Included
