#include "ClipPlanesDialog.h"
#include <gui/TransformSettings.h>
#include <gui/FloatSlider.h>
#include <gui/ApplicationWindow.h>
#include <osg/ClipPlane>
#include <osg/ClipNode>
#include <QtGui>


//----------------------------------------------------------
ClipPlanesDialog::ClipPlanesDialog(Viewer* viewer, ApplicationWindow* parent) 
  : QDialog(parent),
    viewer(viewer)
//----------------------------------------------------------

{
  TransformSettings* ts = viewer->transformSettings();

  layout = new QVBoxLayout(this);

//   // View direction buttons
//   cameraGroup = new QGroupBox("Camera", this);
//   cameraGroupLayout = new QHBoxLayout(cameraGroup);
//   cameraGroupLayout->setSpacing(6);
//   cameraGroupLayout->setMargin(6);

//   // Axial view
//   ax_view_button = new QPushButton("Axial", cameraGroup);
//   cameraGroupLayout->addWidget(ax_view_button);

//   // Coronary view
//   cor_view_button = new QPushButton("Coronary", cameraGroup);
//   cameraGroupLayout->addWidget(cor_view_button);

//   // Sagial view
//   sag_view_button = new QPushButton("Sagial", cameraGroup);
//   cameraGroupLayout->addWidget(sag_view_button);
//   layout->addWidget(cameraGroup);

  //Clip planes
  clip_x_chk = new QCheckBox("Clip X plane", this);
  clip_x_chk->setChecked(ts->isClipped_x());
  layout->addWidget(clip_x_chk);
  
  clip_x_slider = new FloatSlider(this);
  clip_x_slider->setLabel(tr("Clip X plane"));
  clip_x_slider->setMinValue(-250);
  clip_x_slider->setMaxValue(250);
  clip_x_slider->setEnabled(clip_x_chk->isChecked());
  clip_x_slider->setCurrentValue(ts->clipValue_x());
  layout->addWidget(clip_x_slider);

  clip_y_chk = new QCheckBox("Clip Y plane", this);
  clip_y_chk->setChecked(ts->isClipped_y());
  layout->addWidget(clip_y_chk);

  clip_y_slider = new FloatSlider(this);
  clip_y_slider->setLabel(tr("Clip Y plane"));
  clip_y_slider->setMinValue(-250);
  clip_y_slider->setMaxValue(250);
  clip_y_slider->setEnabled(clip_y_chk->isChecked());
  clip_y_slider->setCurrentValue(ts->clipValue_y());
  layout->addWidget(clip_y_slider);
  
  clip_z_chk = new QCheckBox("Clip Z plane", this);
  clip_z_chk->setChecked(ts->isClipped_z());
  layout->addWidget(clip_z_chk);
	
  clip_z_slider = new FloatSlider(this);
  clip_z_slider->setLabel(tr("Clip Z plane"));
  clip_z_slider->setMinValue(-250);
  clip_z_slider->setMaxValue(250);
  clip_z_slider->setEnabled(clip_z_chk->isChecked());
  clip_z_slider->setCurrentValue(ts->clipValue_z());
  layout->addWidget(clip_z_slider);
  
  
  verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);
  layout->addItem(verticalSpacer);

  closeButton = new QPushButton(tr("Close"));
  layout->addWidget(closeButton);

  /*************************** Connect widgets ********************************/

  connect(closeButton, SIGNAL(clicked()),
	  this, SLOT(accept()));


  connect(clip_x_chk, SIGNAL(clicked()),
	  this, SLOT(clipPlaneX()));
  connect(clip_x_slider, SIGNAL(valueChanged(double)),
	  this, SLOT(clipPlaneX()));
	  
  connect(clip_y_chk, SIGNAL(clicked()),
	  this, SLOT(clipPlaneY()));
  connect(clip_y_slider, SIGNAL(valueChanged(double)),
	  this, SLOT(clipPlaneY()));

  connect(clip_z_chk, SIGNAL(clicked()),
	  this, SLOT(clipPlaneZ()));
  connect(clip_z_slider, SIGNAL(valueChanged(double)),
	  this, SLOT(clipPlaneZ()));
}
//----------------------------------------------------------
ClipPlanesDialog::~ClipPlanesDialog() 
//----------------------------------------------------------
{
  //Disconnect everything signaling to this object
  disconnect(closeButton, 0,this,0);
  disconnect(clip_x_chk, 0, this, 0);
  disconnect(clip_x_slider, 0, this, 0);
  disconnect(clip_y_chk, 0, this, 0);
  disconnect(clip_y_slider, 0, this, 0);
  disconnect(clip_z_chk, 0, this, 0);
  disconnect(clip_z_slider, 0, this, 0);
}
//----------------------------------------------------------
void ClipPlanesDialog::clipPlaneX() 
//----------------------------------------------------------
{
  
  clip_x_slider->setEnabled(clip_x_chk->isChecked());
  viewer->transformSettings()->clipPlaneX(clip_x_chk->isChecked(),
					  clip_x_slider->getValue());

}

//----------------------------------------------------------
void ClipPlanesDialog::clipPlaneY() 
//----------------------------------------------------------
{
  clip_y_slider->setEnabled(clip_y_chk->isChecked());
  viewer->transformSettings()->clipPlaneY(clip_y_chk->isChecked(),
					  clip_y_slider->getValue());
}

//----------------------------------------------------------
void ClipPlanesDialog::clipPlaneZ() 
//----------------------------------------------------------
{
  clip_z_slider->setEnabled(clip_z_chk->isChecked());
  viewer->transformSettings()->clipPlaneZ(clip_z_chk->isChecked(),
					  clip_z_slider->getValue());
}
