#ifndef CLIP_PLANES_DIALOG
#define CLIP_PLANES_DIALOG

#include <QDialog>

class ApplicationWindow;
class QCheckBox;
class QPushButton;
class QVBoxLayout;
class QHBoxLayout;
class QGroupBox;
class QSpacerItem;
class FloatSlider;
class Viewer;

namespace osg{ class ClipPlane; }

/**
 * A dialog for setting clipping planes. 
 * The object will connect to the current viewers 
 * TransformSettings when created and disconnect
 * when destroyed. The clipping planes are updated
 * interactively.
 * 
 * \author Benjamin Kehlet
 * \date 2010-03-27
 * \since 2010-03-27
 */
class ClipPlanesDialog : public QDialog {
  Q_OBJECT

public:
  ClipPlanesDialog(Viewer* viewer, ApplicationWindow* parent=0);
  ~ClipPlanesDialog();

protected slots:
  void clipPlaneX();
  void clipPlaneY();
  void clipPlaneZ();


 private:
  QVBoxLayout *layout;

/*   QGroupBox      *cameraGroup; */
/*   QHBoxLayout    *cameraGroupLayout; */
/*   QPushButton    *ax_view_button; */
/*   QPushButton    *cor_view_button; */
/*   QPushButton    *sag_view_button; */

  QCheckBox      *clip_x_chk;
  FloatSlider    *clip_x_slider;
  osg::ClipPlane *clip_x_plane;

  QCheckBox      *clip_y_chk;
  FloatSlider    *clip_y_slider;
  osg::ClipPlane *clip_y_plane;

  QCheckBox      *clip_z_chk;
  FloatSlider    *clip_z_slider;
  osg::ClipPlane *clip_z_plane;

  QSpacerItem *verticalSpacer;
  QPushButton *closeButton;
  Viewer      *viewer;
    
};

#endif
