#include <gui/ColorButton.h>
#include <ui_ColorButton.h>

#include <QColorDialog>
#include <QPushButton>

//----------------------------------------------------------
ColorButton::ColorButton(QWidget* parent, Qt::WindowFlags f)
  : QWidget(parent, f),
    ui_(new Ui::ColorButtonBase),
    use_alpha_(true)
//----------------------------------------------------------
{
  ui_->setupUi(this);
  
  setColor(qRgba(255, 255, 255, 255));
  
  connect(ui_->button_, SIGNAL(clicked()), this, SLOT(colorButtonPressed()));
}

//----------------------------------------------------------
ColorButton::~ColorButton()
//----------------------------------------------------------
{
  delete ui_;
}

//----------------------------------------------------------
void ColorButton::setColor(const QRgb color)
//----------------------------------------------------------
{
  color_ = color;
  
  QString style = QString("background: %1").arg(QColor(color).name());
  ui_->button_->setStyleSheet(style);
}

  
//----------------------------------------------------------
void ColorButton::setUseAlpha(bool use_alpha)
//----------------------------------------------------------
{
  use_alpha_ = use_alpha;
}

//----------------------------------------------------------
bool ColorButton::getUseAlpha() const
//----------------------------------------------------------
{
  return use_alpha_;
}
  
//----------------------------------------------------------
void ColorButton::colorButtonPressed()
//----------------------------------------------------------
{
  bool ok;
  QRgb new_color;
    
  if(use_alpha_)
    new_color = QColorDialog::getRgba(color_, &ok);
  else {
    QColor c = QColorDialog::getColor(color_);
      
    ok=c.isValid();
      
    if(ok)
      new_color = qRgb(c.red(), c.green(), c.blue());
  }

  if(ok && new_color != color_) {
    setColor(new_color);
    emit colorChanged(color_);
    emit colorChanged(QColor(color_));
  }
}
  
//----------------------------------------------------------
const QRgb &ColorButton::getRgb() const
//----------------------------------------------------------
{
  return color_;
}

//----------------------------------------------------------
QColor ColorButton::getColor() const
//----------------------------------------------------------
{
  return QColor(color_);
}

//----------------------------------------------------------
void ColorButton::setColor(QColor color)
//----------------------------------------------------------
{
  if (use_alpha_)
    color_ = color.rgba();
  else
    color_ = color.rgb();
  setColor(color_);
}

//----------------------------------------------------------
void ColorButton::setLabel(const QString label)
//----------------------------------------------------------
{
  ui_->text_label_->setText(label);
}
  
//----------------------------------------------------------
QString ColorButton::getLabel() const
//----------------------------------------------------------
{
  return ui_->text_label_->text();
}

