#include <gui/DolfinEventHandler.h>
#include <gui/MeshFunctionsToolbar.h>
#include <gui/Context.h>

#include <scenegraph/OsgDolfinObject.h>

#include <osgUtil/IntersectVisitor>
#include <osg/LineSegment>
#include <osg/Material>
#include <osg/ClearNode>
#include <osg/LightModel>
#include <osg/Light>
#include <osg/PolygonMode>

#include <osgQt/TrackballManipulator.h>

#include <QDebug>

//----------------------------------------------------------
DolfinEventHandler::DolfinEventHandler(MeshFunctionsToolbar* toolbar)
  : viewer(0), toolbar(toolbar)
//----------------------------------------------------------
{
  std::cerr << "DolfinEventHandler::constructor" << std::endl;
}

//----------------------------------------------------------
void DolfinEventHandler::setViewer(Viewer* viewer)
//----------------------------------------------------------
{
  std::cout << "DolfinEventHandler, setting viewer to " << viewer << std::endl;
  std::cout << "DolfinObject at " << &viewer->get_mesh() << std::endl;
  this->viewer = viewer;
}

//----------------------------------------------------------
Viewer* DolfinEventHandler::getViewer() const
//----------------------------------------------------------
{
  return viewer; 
}
//----------------------------------------------------------
bool DolfinEventHandler::handle(const osgGA::GUIEventAdapter& ea,
				osgGA::GUIActionAdapter& aa)
//----------------------------------------------------------
{
  //std::cerr << "DolfinEventHandler::handle" << std::endl;
  
  // Check for regular events (i.e. any mouse mode)
  switch(ea.getEventType()) {
    case osgGA::GUIEventAdapter::PUSH :
    {
      //qDebug() << "DolfinEventHandler: button push: " << ea.getButton() << " " << ea.getModKeyMask();
      return mousePressEvent(ea, aa);
    }
    break;
    
    case osgGA::GUIEventAdapter::DRAG:
    {
      //qDebug() << "DolfinEventHandler: drag";
      return mouseMoveEvent(ea, aa);
    }
    break;

    case osgGA::GUIEventAdapter::MOVE:
    {
      //qDebug() << "DolfinEventHandler: move";
      return mouseMoveEvent(ea, aa);
    }
    break;
    
    case osgGA::GUIEventAdapter::RELEASE:
    {
      //qDebug() << "DolfinEventHandler: release";
      return mouseReleaseEvent(ea, aa);
    }
    break;
    
    case osgGA::GUIEventAdapter::KEYDOWN: 
    {
      return keyPressEvent(ea, aa);
    }
    break;

    default :
      return false;
  }
}
    
//----------------------------------------------------------
bool DolfinEventHandler::mousePressEvent(const osgGA::GUIEventAdapter& ea,
					 osgGA::GUIActionAdapter& aa)
//----------------------------------------------------------
{
  if (ea.getButton()==osgGA::GUIEventAdapter::LEFT_MOUSE_BUTTON) {
    setDolfinBoundaryIndicator(ea, aa);
    return true;
  }
        
  return false;
}
    
//----------------------------------------------------------
bool DolfinEventHandler::mouseMoveEvent(const osgGA::GUIEventAdapter& ea,
					osgGA::GUIActionAdapter& aa)
//----------------------------------------------------------
{
  setDolfinBoundaryIndicator(ea, aa);    
  return true;
}
    
//----------------------------------------------------------
bool DolfinEventHandler::mouseReleaseEvent(const osgGA::GUIEventAdapter& ea,
					   osgGA::GUIActionAdapter& aa)
//----------------------------------------------------------
{
  dolfinBoundaryIndicatorReleased();
  return true;
      
  //viewer_->updateGL();
  //return false;
}
    
//----------------------------------------------------------
bool DolfinEventHandler::keyPressEvent(const osgGA::GUIEventAdapter& ea,
				       osgGA::GUIActionAdapter& aa)
//----------------------------------------------------------
{      
  OsgTransformObject *curr_obj = viewer->getOsgObj();
  
  if (!curr_obj)
    return false;
  
  if(ea.getKey() == 'c' && curr_obj) 
  {
    viewer->setTrackballCenter(curr_obj);
  }
  else if(ea.getKey() == 'p' && curr_obj) 
  {
    const osg::Vec3 &trans    = curr_obj->getTranslation();
    //const osg::Vec3 &scale    = curr_obj->getScale();
    const osg::Vec3 &rot_axis = curr_obj->getRotationAxis();
    const double   &rot_angle = curr_obj->getRotationAngle();
    
    std::cerr << "curr_node->getInfo: rot_angle, " << 180.0*rot_angle/M_PI
              << ", rot_axis: " << rot_axis.x() << " " << rot_axis.y() << " "
              << rot_axis.z() << std::endl;
    
    std::cerr << "Trans: " << trans.x() << " " 
              << trans.y() << " " 
              << trans.z() << std::endl;
    
    osg::Matrix mat = curr_obj->getMatrix();
    std::cerr << "Mat:" << std::endl;
    for (int i=0; i<4; i++) 
    {
      for (int j=0; j<4; j++)
      {
        std::cerr << mat(i,j) << " ";
      }
      std::cerr << std::endl;
    }
  }
  
  viewer->updateGL();
  
  return false;
}
    
//----------------------------------------------------------
void DolfinEventHandler::getUsage(osg::ApplicationUsage& usage) const
//----------------------------------------------------------
{
}

//----------------------------------------------------------
void DolfinEventHandler::setDolfinBoundaryIndicator(const osgGA::GUIEventAdapter& ea,
                                                    osgGA::GUIActionAdapter& aa)
//----------------------------------------------------------
{

  /** Todo: Rewrite this.
   *  The IntersectVisitor class is depreceated and should be ported to
   *  use Intersector and IntersectionVisitor instead.
   */


  //std::cerr << "DolfinEventHandler::setDolfinBoundaryIndicator" << std::endl;

  // Picking objects
  osg::Vec3 near_point, far_point;
  viewer->projectWindowXYIntoObject(ea.getX(), ea.getY(), near_point, far_point);
    
  osg::ref_ptr<osg::LineSegment> line_segment = new osg::LineSegment(near_point,far_point);

    
  // Create the IntersectVisitor to do the line intersection traversals.
  osgUtil::IntersectVisitor intersector;
  intersector.addLineSegment(line_segment.get());
    
  osg::Group* scene = viewer->getRootNode();
  scene->accept(intersector);
  
  osgUtil::IntersectVisitor::HitList& hits=intersector.getHitList(line_segment.get());
  
  if(!hits.empty()) 
  {
    osgUtil::Hit& hit = hits.front();
    
    osg::NodePath& node_path = hit._nodePath;
    //osg::Node* node = (node_path.size()>=1) ? node_path[node_path.size()-1] : 0;
    //osg::Group* parent = (node_path.size()>=2) ? dynamic_cast<osg::Group*>(node_path[node_path.size()-2]) : 0;
    OsgDolfinObject* dobj = 0;
    
    for(unsigned int i = 0; i < node_path.size(); i++)  
    {
      if(!dobj)
        dobj = dynamic_cast<OsgDolfinObject*>(node_path[i]);
    }
    
    // Dolfin object hit
    if(dobj) 
      setBndryIndctr(dobj, hit);
  }
}
//----------------------------------------------------------
void DolfinEventHandler::setBndryIndctr(OsgDolfinObject *osgDolfin_object, osgUtil::Hit& hit)
//----------------------------------------------------------
{
  const osgUtil::Hit::VecIndexList& list = hit.getVecIndexList();

  //cerr << "MeshFunctions::setDolfinBoundaryIndicator(): Called, list.size = " << list.size() << endl;

  if(list.size() != 3) // Return if we hit something that has something else than 3 vertices
    return;
  
  // We hit something relevant, so make a undo copy of the current set of data first, 
  // if this is the beginning of a new "draw move". Don't make a copy if we're dragging
  // the mouse as that would be too often.
  
  // Find barycentric coordinates of intersection point.
  osg::Vec3 p = hit.getLocalIntersectPoint();
  osg::Vec3 v[3];
  for(int k=0 ; k < 3 ; k++)
    v[k] = (*osgDolfin_object->getVertexArray())[list[k]];
  
  double bary[3];
  double Atot = ((v[2]-v[0])^(v[1]-v[0])).length();
  double A[3];
  for(int k = 0; k < 3; k++) 
  {
    A[k] = ((v[(k+1)%3]-p)^(v[(k+2)%3]-p)).length();
    bary[k] = A[k]/Atot;
  }
    
  //=========================
  // Vertex paint functions
  //=========================
  if (toolbar->getEntityMode() == MeshFunctionsToolbar::VERTEX) 
  {
    // Find the closest vertex to the mouse point. 
    // This is the one with the highest barycentric coordinate value
    int max_index = 0;
    for (int index = 1; index < 3; ++index) 
    {
      if(bary[max_index] < bary[index])
        max_index = index;
    }

    viewer->get_mesh().setVertexBoundaryIndicator(list[max_index], 
                                                 toolbar->getFunctionValue(), 
                                                 toolbar->getSelectMode());
  }
  else if (toolbar->getEntityMode() == MeshFunctionsToolbar::EDGE) 
  {
    //=========================
    // Edge paint functions
    //=========================

    // Figure out which edge is closest
    // First find the point that is furthest away
    int min_index = 0;
    for(int index = 1; index < 3; ++index) 
    {
      if(bary[min_index] > bary[index])
        min_index = index;
    }

    // Then use the two other ones which defines the edge we want
    int i1, i2;
    if(min_index == 0) 
    {
      i1 = 1;
      i2 = 2;
    }
    else if(min_index == 1) 
    {
      i1 = 0;
      i2 = 2;
    }
    else if(min_index == 2) 
    {
      i1 = 0;
      i2 = 1;
    }
	
    // and draw with those
    uint edgeid = osgDolfin_object->findEdgeFromVertices(list[i1], list[i2]);
    
    viewer->get_mesh().setEdgeBoundaryIndicator(edgeid, 
                                               toolbar->getFunctionValue(),
                                               toolbar->getSelectMode());
  }
  else 
  {
    //=========================
    // Face paint functions
    //=========================

    uint faceid = osgDolfin_object->findFaceFromVertices(list[0], list[1], list[2]);
    viewer->get_mesh().setFaceBoundaryIndicator(faceid, 
                                               toolbar->getFunctionValue(),
                                               toolbar->getSelectMode());
  }

  viewer->requestRedraw();
}

//----------------------------------------------------------
void DolfinEventHandler::dolfinBoundaryIndicatorReleased()
//----------------------------------------------------------
{
  is_drawing = false;
}

