#ifndef Gui_DolfinEventHandler_h_Included   // -*- C++ -*-
#define Gui_DolfinEventHandler_h_Included

#include <gui/Viewer.h>

#include <osg/CameraNode>

#include <osgGA/GUIEventHandler>
#include <osgManipulator/CommandManager>
#include <osgManipulator/TabBoxDragger>
#include <osgManipulator/RotateSphereDragger>
#include <osgManipulator/Selection>

class ApplicationWindow;
class OsgDolfinObject;
class MeshFunctionsToolbar;

/**
 * Handles the gui events related to MeshFunctions (and should probably be 
 * renamed to MeshFunctionsEventHandler or so).
 * Note that it inherits from osgGA:GUIEventHandler and
 * therefore uses OSG events, not Qt events. These are fired
 * from the AdapterWidget class.
 * 
 * \author Yvon Halbwachs
 * \author Benjamin Kehlet
 *
 * \date 2010-04-07
 */
    
class DolfinEventHandler : public osgGA::GUIEventHandler
{
public:
  
  
  DolfinEventHandler(MeshFunctionsToolbar* toolbar);
  ~DolfinEventHandler() {}
  
  
  virtual void accept(osgGA::GUIEventHandlerVisitor& v) { v.visit(*this); }
  
  virtual bool handle(const osgGA::GUIEventAdapter& ea,osgGA::GUIActionAdapter&);
  
  virtual void getUsage(osg::ApplicationUsage& usage) const;
  
  void setViewer(Viewer* viewer);
  
  Viewer* getViewer() const;

 protected:
  
  DolfinEventHandler(const DolfinEventHandler&,const osg::CopyOp&) {}
  
  virtual bool mousePressEvent(const osgGA::GUIEventAdapter& ea,
			       osgGA::GUIActionAdapter& aa);
  
  virtual bool mouseMoveEvent(const osgGA::GUIEventAdapter& ea,
			      osgGA::GUIActionAdapter& aa);
  
  virtual bool mouseReleaseEvent(const osgGA::GUIEventAdapter& ea,
				 osgGA::GUIActionAdapter& aa);
  
  virtual bool keyPressEvent(const osgGA::GUIEventAdapter& ea,
			     osgGA::GUIActionAdapter& aa);
  
  void setDolfinBoundaryIndicator(const osgGA::GUIEventAdapter& ea,
				  osgGA::GUIActionAdapter& aa);
 private:
  Viewer *viewer;
  MeshFunctionsToolbar* toolbar;

  /** Moved from old Meshfunctions class. Should probably not be here...
   */
  void setBndryIndctr(OsgDolfinObject *osgDolfin_object, osgUtil::Hit& hit);
  void dolfinBoundaryIndicatorReleased();
  bool is_drawing;
};

#endif // Gui_DolfinEventHandler_h_Included
