#ifndef Gui_DOLFINOBJECT_H             // -*- C++ -*-
#define Gui_DOLFINOBJECT_H

#include <dolfin/mesh/MeshFunction.h>
#include <dolfin/mesh/Mesh.h>

#include <QString>

class ViewerInterface;

/**
 * Contains Dolfin mesh, mesh data
 * Provides an interface to the mesh for the viewer classes
 * and takes care of the mapping from the surface mesh to 
 * surface mesh.
 *
 * \ingroup scenegraph
 *
 * \author Benjamin Kehlet
 * \since 2011-01-30
 * \date 2011-01-30
 *
 */

class DolfinObject : public dolfin::Mesh
{
 public:
  DolfinObject(QString filename);
  ~DolfinObject();

  enum SELECT_MODE{SINGLE, FILL, ALL};

  void loadFaceBoundaryIndicators(QString filename);
  void loadEdgeBoundaryIndicators(QString filename);
  void loadVertexBoundaryIndicators(QString filename);
  
  void saveFaceBoundaryIndicators(QString filename);
  void saveEdgeBoundaryIndicators(QString filename);
  void saveVertexBoundaryIndicators(QString filename);

  // Vertex boundary indicator functions
  void setVertexBoundaryIndicator(uint id, int value, SELECT_MODE mode, bool redraw=true);
  int  getVertexBoundaryIndicator(uint id);
  
  // Edge boundary indicator functions
  void setEdgeBoundaryIndicator(uint id, int value, SELECT_MODE mode, bool redraw=true);
  int  getEdgeBoundaryIndicator(uint id);

  
  // Face boundary indicator functions
  void setFaceBoundaryIndicator(uint id, int value, SELECT_MODE mode, bool redraw=true);
  int  getFaceBoundaryIndicator(uint id);

  void undo();
  void updateUndoBuffers();

  const dolfin::Mesh& surface_mesh() { return *surface_mesh_; }

  void addViewer(ViewerInterface* v);

 private:
  void recursiveVertexFill(uint id, int value);
  void recursiveEdgeFill(uint id, int value);
  void recursiveFaceFill(uint id, int value);

  
  // The dolfin surface mesh
  dolfin::Mesh* surface_mesh_;
  
  // Meshfunctions to store integers on the mesh
  dolfin::MeshFunction<uint> vertex_boundary_indicators_;
  dolfin::MeshFunction<uint> edge_boundary_indicators_;
  dolfin::MeshFunction<uint> face_boundary_indicators_;
  
  // and the backup version for "undo"
  dolfin::MeshFunction<uint> vertex_undo_indicators_;
  dolfin::MeshFunction<uint> edge_undo_indicators_;
  dolfin::MeshFunction<uint> face_undo_indicators_;
  
  // Meshfunctions for mapping between the mesh and the surface mesh
  dolfin::MeshFunction<unsigned int>* dolfin_vertices_;
  dolfin::MeshFunction<unsigned int> dolfin_edges_;
  dolfin::MeshFunction<unsigned int>* dolfin_faces_;

  // Should this be a pointer. Is there any reason to possibly
  // support more viewers of the same mesh?
  // It should then be possible to remove viewers as well.
  std::vector<ViewerInterface*> viewers;
};

#endif
