#include <gui/DrawSettingsDialog.h>
#include <scenegraph/OsgDolfinObject.h>
#include <gui/ApplicationWindow.h>
#include <QtGui>
#include <iostream>

//----------------------------------------------------------
DrawSettingsDialog::DrawSettingsDialog(Viewer* viewer, ApplicationWindow* parent)
  : QDialog(parent), viewer(viewer)
//----------------------------------------------------------
{
  QVBoxLayout* mainLayout = new QVBoxLayout;
  setLayout(mainLayout);

  QWidget* container = new QWidget(this);
  QGridLayout* layout = new QGridLayout;
  container->setLayout(layout);

  //Vertex
  QLabel* vertex_label = new QLabel("Vertex size:");
  layout->addWidget(vertex_label, 0,0);

  vertex_size = new QSpinBox(this);
  layout->addWidget(vertex_size, 0,1);
  
  //Edge
  QLabel* edge_label = new QLabel("Edge size:");
  layout->addWidget(edge_label, 1, 0);

  edge_size = new QSpinBox(this);
  layout->addWidget(edge_size, 1, 1);

  mainLayout->addWidget(container);

  QPushButton* close_button = new QPushButton("Close");
  mainLayout->addWidget(close_button);

  OsgDolfinObject* osg_obj = dynamic_cast<OsgDolfinObject*>(viewer->getOsgObj());

  vertex_size->setValue(osg_obj->getVertexSize());
  edge_size->setValue(osg_obj->getEdgeSize());


  connect(vertex_size, SIGNAL(valueChanged(int)),
	  this, SLOT(updateSettings()));

  connect(edge_size, SIGNAL(valueChanged(int)),
	  this, SLOT(updateSettings()));

  connect(close_button, SIGNAL(clicked()),
	  this, SLOT(accept()));

}
//----------------------------------------------------------
DrawSettingsDialog::~DrawSettingsDialog()
//----------------------------------------------------------
{}
//----------------------------------------------------------
void DrawSettingsDialog::updateSettings() 
//----------------------------------------------------------
{
  OsgDolfinObject* osg_obj = dynamic_cast<OsgDolfinObject*> (viewer->getOsgObj());

  osg_obj->setVertexSize(vertex_size->value());
  osg_obj->setEdgeSize(edge_size->value());
  osg_obj->updateMaterial();
    
  viewer->requestRedraw();
}
