#include "Edge_2D.h"
#include "Scene_2D.h"
#include <QGraphicsSceneMouseEvent>
#include <QPen>
#include <QPainter>
#include <QDebug>

Edge_2D::Edge_2D(double x1, double y1, double x2, double y2, uint id) 
  : EntityInterface_2D(id), line(x1,  y1, x2, y2), 
    size(0.01), normal_pen(Qt::gray, 0), hovered_pen(Qt::gray, 0.01)
{

}
//----------------------------------------------------------
QRectF Edge_2D::boundingRect() const
//----------------------------------------------------------
{
  return shape().boundingRect();
}
//----------------------------------------------------------
QPainterPath Edge_2D::shape() const
//----------------------------------------------------------
{
  QPainterPath path;

  path.moveTo(line.p1());
  path.lineTo(line.p2());

  return path;
}
//----------------------------------------------------------
void Edge_2D::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *)
//----------------------------------------------------------
{
  painter->setPen(hovered ? hovered_pen : normal_pen);
  painter->drawLine(line);
}
//----------------------------------------------------------
void Edge_2D::setMeshFunctionValue(uint value, QColor& c)
//----------------------------------------------------------
{
  if (this->value != value)
  {
    // set the value in the mesh
    static_cast<Scene_2D*>(scene())->get_mesh().setEdgeBoundaryIndicator(id, value, DolfinObject::SINGLE);

    //set the pen color
    hovered_pen.setColor(c);
    normal_pen.setColor(c);
    
    //schedule a repainting with the new color
    update();
  }  
}
//----------------------------------------------------------
void Edge_2D::updateBoundaryIndicatorColor()
//----------------------------------------------------------
{
  Scene_2D* s =  static_cast<Scene_2D*>(scene());
  uint mesh_value = s->get_mesh().getEdgeBoundaryIndicator(id);
  if (mesh_value != value)
  {
    QColor c = s->get_value_color(mesh_value);
    normal_pen.setColor(c);
    hovered_pen.setColor(c);

    update();
  }
}
