#include <gui/FloatSlider.h>
#include <ui_FloatSlider.h>

#include <QSlider>
#include <QLineEdit>

//----------------------------------------------------------
FloatSlider::FloatSlider(QWidget* parent, Qt::WindowFlags f)
  : QWidget(parent, f),
    ui_(new Ui::FloatSliderBase),
    min_value_(0.0),
    max_value_(1.0),
    granularity_(0.01),
    value_(0.0),
    slider_value_(0)
//----------------------------------------------------------
{
  ui_->setupUi(this);

    
  ui_->line_edit_->setAlignment(Qt::AlignRight);
    
  connect(ui_->slider_, SIGNAL(valueChanged(int)),
	  this, SLOT(sliderValueChanged(int)));
    
  connect(ui_->line_edit_, SIGNAL(returnPressed()),
	  this, SLOT(textChanged()));
    
  connect(ui_->line_edit_, SIGNAL(lostFocus()),
	  this, SLOT(textChanged()));
    
  updateSliderRange();
}
  
//----------------------------------------------------------
FloatSlider::~FloatSlider()
//----------------------------------------------------------
{
}
  
//----------------------------------------------------------
void  FloatSlider::setMinValue(double v)
//----------------------------------------------------------
{
  min_value_ = v;
  updateSliderRange();
}
  
//----------------------------------------------------------
void  FloatSlider::setMaxValue(double v)
//----------------------------------------------------------
{
  max_value_ = v;
  updateSliderRange();
}
  
//----------------------------------------------------------
void  FloatSlider::setGranul(double g)
//----------------------------------------------------------
{
  granularity_ = g;
  updateSliderRange();
}
  
//----------------------------------------------------------
void FloatSlider::setCurrentValue(double v)
//----------------------------------------------------------
{
  if(value_ != v) {
    value_ = v;
    emit valueChanged(value_);
  }
  updateWidgets();
    
}
  
//----------------------------------------------------------
void FloatSlider::sliderValueChanged(int v)
//----------------------------------------------------------
{
  double tmp = min_value_ + v*(max_value_-min_value_)/no_slider_steps_;
    
  if(value_ != tmp) {
    value_ = tmp;
    emit valueChanged(value_);
  }
  updateWidgets();
}
  
//----------------------------------------------------------
void FloatSlider::textChanged()
//----------------------------------------------------------
{
  QString s = ui_->line_edit_->text();
  bool ok;
  double tmp = s.toDouble(&ok);
    
  if(ok && value_ != tmp) {
    value_ = tmp;
    emit valueChanged(value_);
  }
  updateWidgets();
}
  
//----------------------------------------------------------
void FloatSlider::updateSliderRange()
//----------------------------------------------------------
{
  no_slider_steps_=(int)((max_value_-min_value_)/granularity_+0.5);
    
  ui_->slider_->setRange(0, no_slider_steps_);
    
  updateWidgets();
}
  
//----------------------------------------------------------
void FloatSlider::updateWidgets()
//----------------------------------------------------------
{
  QString le_val = QString::number(value_);
    
  if(le_val != line_edit_value_) {
    line_edit_value_ = le_val;
	
    ui_->line_edit_->blockSignals(true);
    ui_->line_edit_->setText(line_edit_value_);
    ui_->line_edit_->blockSignals(false);
  }
    
  int sl_val=0;
  if(max_value_ > min_value_)
    sl_val = (int)((value_-min_value_) * no_slider_steps_ /
		   (max_value_-min_value_)+0.5);
    
  if(sl_val < 0 ) sl_val=0;
  if(sl_val > no_slider_steps_) sl_val=no_slider_steps_;
    
  if(sl_val != slider_value_) {
    slider_value_ = sl_val;
      
    ui_->slider_->blockSignals(true);
    ui_->slider_->setValue(slider_value_);
    ui_->slider_->blockSignals(false);
  }
}

//----------------------------------------------------------
double FloatSlider::getValue() const
//----------------------------------------------------------
{
  return value_;
}
  
//----------------------------------------------------------
double FloatSlider::getMinValue() const
//----------------------------------------------------------
{
  return min_value_;
}
  
//----------------------------------------------------------
double FloatSlider::getMaxValue() const
//----------------------------------------------------------
{
  return max_value_;
}
  
//----------------------------------------------------------
double FloatSlider::getGranul() const
//----------------------------------------------------------
{
  return granularity_;    
}

//----------------------------------------------------------
void FloatSlider::setLabel(const QString label)
//----------------------------------------------------------
{
  ui_->label_->setText(label);
}
  
//----------------------------------------------------------
QString FloatSlider::getLabel() const
//----------------------------------------------------------
{
  return ui_->label_->text();
}

