#include "MeshFunctionsToolbar.h"

#include <gui/Viewer.h>
#include <gui/Context.h>
#include <gui/ApplicationWindow.h>
#include <gui/DolfinEventHandler.h>
#include <gui/Colormap.h>
#include <scenegraph/OsgDolfinObject.h>


#include <QButtonGroup>
#include <QPushButton>
#include <QSpinBox>
#include <QCheckBox>
#include <QLabel>
#include <QComboBox>
#include <QDebug>

//----------------------------------------------------------
MeshFunctionsToolbar::MeshFunctionsToolbar(Context &context)
  : context(context), QToolBar(context.app())
//----------------------------------------------------------
{
  on_off_button = new QPushButton("Meshfunctions");
  on_off_button->setCheckable(true);
  addWidget(on_off_button);
  connect(on_off_button, SIGNAL(toggled(bool)),
	  this, SLOT(enable(bool)));
  addSeparator();

  function_value = new QSpinBox(this);
  function_value->setValue(1);
  addWidget(function_value);

  addSeparator();

  button_group = new QButtonGroup(this);

  vertex = new QPushButton("Vertex", this);
  vertex->setCheckable(true);
  vertex->setChecked(true);
  addWidget(vertex);
  button_group->addButton(vertex);

  edge   = new QPushButton("Edge", this);
  edge->setCheckable(true);
  addWidget(edge);
  button_group->addButton(edge);

  face   = new QPushButton("Face", this);
  face->setCheckable(true);
  addWidget(face);
  button_group->addButton(face);

  addSeparator();


  markMethod = new QComboBox(this);
  markMethod->addItem("Single");
  markMethod->addItem("Fill");
  markMethod->addItem("All");
  addWidget(markMethod);

  addSeparator();

  color_map_button = new QPushButton("Show colormap", this);
  color_map_button->setCheckable(true);
  addWidget(color_map_button);

  color_map = new ColorMap();


  connect(color_map_button, SIGNAL(toggled(bool)),
	  this, SLOT(show_color_map(bool)));

  // connect in case the user close the dialog window
  connect(color_map, SIGNAL(finished(int)),
	  this, SLOT(close_color_map()));


  //Initialize state
  enable(on_off_button->isChecked());
}
//----------------------------------------------------------
void MeshFunctionsToolbar::enable(bool enable)
//----------------------------------------------------------
{
  function_value->setEnabled(enable);

  vertex->setEnabled(enable);
  edge->setEnabled(enable);
  face->setEnabled(enable);

  close_color_map();
  color_map_button->setEnabled(enable);
  markMethod->setEnabled(enable);

  context.enableMeshFunctions(enable);
  //viewer->enableMeshFunctions(enable);

}
//----------------------------------------------------------
void MeshFunctionsToolbar::show_color_map(bool show)
//----------------------------------------------------------
{
  color_map_button->blockSignals(true);
  color_map_button->setChecked(show);
  color_map_button->blockSignals(false);
  color_map->setVisible(show);
}
//----------------------------------------------------------
void MeshFunctionsToolbar::close_color_map() 
//----------------------------------------------------------
{
  show_color_map(false);
}
//----------------------------------------------------------
MeshFunctionsToolbar::ENTITY_MODE MeshFunctionsToolbar::getEntityMode()
//----------------------------------------------------------
{
  if (face->isChecked())   { return FACE;   }
  if (edge->isChecked())   { return EDGE;   }
  if (vertex->isChecked()) { return VERTEX; }
  Q_ASSERT(false);
}
//----------------------------------------------------------
DolfinObject::SELECT_MODE MeshFunctionsToolbar::getSelectMode()
//----------------------------------------------------------
{
  if (markMethod->currentText() == "Single") return DolfinObject::SINGLE;
  if (markMethod->currentText() == "Fill") return DolfinObject::FILL;
  if (markMethod->currentText() == "All") return DolfinObject::ALL;
  Q_ASSERT(false);
}
//----------------------------------------------------------
int MeshFunctionsToolbar::getFunctionValue()
//----------------------------------------------------------
{
  return function_value->value();
}
