#include "Scene_2D.h"
#include <gui/MeshFunctionsToolbar.h>
#include "Edge_2D.h"
#include "Vertex_2D.h"
#include <QGraphicsSceneMouseEvent>
#include <QDebug>

//----------------------------------------------------------
Scene_2D::Scene_2D(DolfinObject& mesh, QObject* parent)
  : QGraphicsScene(parent), hovered(NULL), mesh(mesh)
//----------------------------------------------------------
{ 
  // Create color palette
  color_palette.push_back(Qt::gray);
  color_palette.push_back(Qt::red);
  color_palette.push_back(Qt::green);
  color_palette.push_back(Qt::blue);
  color_palette.push_back(Qt::yellow);
  color_palette.push_back(Qt::cyan);
  color_palette.push_back(Qt::magenta);
  color_palette.push_back(Qt::white);
  color_palette.push_back(Qt::black);
}
//----------------------------------------------------------
void Scene_2D::mousePressEvent(QGraphicsSceneMouseEvent *event)
//----------------------------------------------------------
{
  //qDebug() << "Mouse pressed at point (" << event->scenePos().x() << ", " << event->scenePos().y() << ")";
}
//----------------------------------------------------------
void Scene_2D::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
//----------------------------------------------------------
{
  //qDebug() << "Mouse moved to point (" << event->scenePos().x() << ", " << event->scenePos().y() << ")";

  if (!markingEnabled) return;

  // Make a rectangle 
  double size = .5;
  QPointF pos = event->scenePos();

  QList<QGraphicsItem *> hits = items(pos.x()-size, pos.y()-size, size*2, size*2);
  if (toolbar->getEntityMode() == MeshFunctionsToolbar::VERTEX)
    hovered = setHovered<Vertex_2D>(pos, hits, hovered);
  else if (toolbar->getEntityMode() == MeshFunctionsToolbar::EDGE)
    hovered = setHovered<Edge_2D>(pos, hits, hovered);

  
  QGraphicsScene::mouseMoveEvent(event);
}
//----------------------------------------------------------
void Scene_2D::dragMoveEvent(QGraphicsSceneDragDropEvent* event)
//----------------------------------------------------------
{
  //qDebug() << "Mouse dragged at point (" << event->scenePos().x() << ", " << event->scenePos().y() << ")";
}
//----------------------------------------------------------
void Scene_2D::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
//----------------------------------------------------------
{
  //qDebug() << "Mouse released at point (" << event->scenePos().x() << ", " << event->scenePos().y() << ")";
  if (markingEnabled && hovered)
  {
    uint value = toolbar->getFunctionValue();
    hovered->setMeshFunctionValue(value, color_palette[value]);
  }
}
//----------------------------------------------------------
void Scene_2D::enableMarking(bool enable, MeshFunctionsToolbar* toolbar)
//----------------------------------------------------------
{
  markingEnabled = enable;

  if (enable)
  {
    this->toolbar = toolbar;
  } 
  else 
  {
    if (toolbar) toolbar = NULL;
    if (hovered)
    {
      Edge_2D* e = dynamic_cast<Edge_2D*>(hovered);
      if (e)
        e->setHovered(false);

      Vertex_2D* v = dynamic_cast<Vertex_2D*>(hovered);
      if (v)
        v->setHovered(false);
    }
  }
}
// //----------------------------------------------------------
// void Scene_2D::change_entity_type(MeshFunctionsToolbar::ENTITY_MODE)
// //----------------------------------------------------------
// {
//   if (hovered)
//   {
//     Edge_2D* e = dynamic_cast<Edge_2D*>(hovered);
//     if (e)
//       e->setHovered(false);

//     Vertex_2D* v = dynamic_cast<Vertex_2D*>(hovered);
//     if (v)
//       v->setHovered(false);
//   }
// }
// void Scene_2D::mousePressEvent(QGraphicsSceneMouseEvent *event)
// {
//   qDebug() << "Mouse pressed";
// }


// void Scene_2D::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
// {
//   qDebug() << "Mouse released";
// }

//----------------------------------------------------------
void Scene_2D::updateBoundaryIndicatorColors()
//----------------------------------------------------------
{
  foreach (QGraphicsItem* item, items())
  {
    static_cast<EntityInterface_2D*>(item)->updateBoundaryIndicatorColor();
  }
}
