#ifndef Gui_SCENE_2D
#define Gui_SCENE_2D


#include <QGraphicsScene>
#include <gui/Edge_2D.h>
#include <gui/Vertex_2D.h>
#include <gui/MeshFunctionsToolbar.h>
#include <gui/DolfinObject.h>


/**
 *  The scene containing the items representing the mesh.  Takes, in
 *  particular care of the mouse events. When clicking the mouse, we
 *  need to search in an region around the cursor. This class takes 
 *  care of the this search.
 *
 * \author Benjamin Kehlet
 * \date  2011-02-20
 * \since 2011-02-18
 */


class Scene_2D : public QGraphicsScene
{
  
 public:
  Scene_2D(DolfinObject& mesh, QObject *parent=0);

  // It is superfluous to send the bool argument since it is contained
  // in the toolbar. Need a minor change to the toolbar interface
  // though.
  void enableMarking(bool enable, MeshFunctionsToolbar* toolbar);
  void updateBoundaryIndicatorColors();

  DolfinObject& get_mesh() { return mesh; }
  QColor get_value_color(uint value) { return color_palette[value]; }

  protected:
  void mousePressEvent(QGraphicsSceneMouseEvent* event); 
  void mouseMoveEvent(QGraphicsSceneMouseEvent* event);
  void mouseReleaseEvent(QGraphicsSceneMouseEvent* event); 
  void dragMoveEvent(QGraphicsSceneDragDropEvent* event);

 private:
  bool markingEnabled;
  MeshFunctionsToolbar* toolbar;
  EntityInterface_2D* hovered;
  DolfinObject& mesh;


  // Note: This is redundant. The color palette should be shared with
  // the 3D viewer.
  QList<QColor> color_palette;

};

template<class ItemClass>
EntityInterface_2D* setHovered(QPointF pos, QList<QGraphicsItem*> hits, EntityInterface_2D* hovered)
{
  ItemClass* closest = NULL;

  if (hits.size() > 0)
  {
    qreal distance = 10e100;
    
    // Find the closest entity of given type
    foreach (QGraphicsItem* hit, hits)
    {
      ItemClass* item = dynamic_cast<ItemClass*>(hit);
      if (!item) continue;
      
      const qreal dist = QLineF(pos, item->sceneBoundingRect().center()).length();
      if (dist < distance) 
      {
        closest = item;
        distance = dist;
      }
    }
  }

  if (closest)
  {
    // Found entity and it is another one than the currently hovered
    if (closest != hovered)
    {
      closest->setHovered(true);
      if (hovered)
        hovered->setHovered(false);
    } 
  } 
  else if (hovered)
  {
    hovered->setHovered(false);
  }

  return closest;
}



#endif
