#include <gui/FloatSlider.h>
#include <gui/ColorButton.h>
#include <gui/Viewer.h>

#include <gui/SimResSettings.h>
#include <scenegraph/OsgDolfinObject.h>

#include <osgGA/TrackballManipulator>

#include <QButtonGroup>
#include <QPushButton>
#include <QCheckBox>
#include <QComboBox>
#include <QLabel>

#include <QDebug>
#include <iostream>

//----------------------------------------------------------
SimResSettings::SimResSettings(Viewer *viewer)
  : viewer(viewer)
//----------------------------------------------------------
{
  std::cout << "SimResSettings: Viewer(" << viewer << ")" << std::endl;
  //setObjectName("SimResSettings");
}

//----------------------------------------------------------
SimResSettings::~SimResSettings()
//----------------------------------------------------------
{
}
//----------------------------------------------------------
void SimResSettings::simresTransparancyChanged(double v)
//----------------------------------------------------------
{
  currState().setTransparency(v);
  viewer->requestRedraw();
}

//----------------------------------------------------------
void SimResSettings::simresRenderModeChanged(int button)
//----------------------------------------------------------
{
  currState().setRenderMode(button);
  /*
  OsgTransformObject *osg_obj = viewer->getOsgObj();
  OsgDrawableObject *drawable = dynamic_cast<OsgDrawableObject*>(osg_obj);
  if (drawable)
    drawable->setCurrentState(currState().get());
  */
  viewer->requestRedraw();
  viewer->updateGL();
  //context_->requestRedraw(true /* UpdateGL */);
}

//----------------------------------------------------------
void SimResSettings::simresCullModeChanged(int mode)
//----------------------------------------------------------
{
    currState().setCullMode(mode);
    viewer->requestRedraw();
}

//----------------------------------------------------------
void SimResSettings::simresTwoSidedLightingChanged(bool on)
//----------------------------------------------------------
{
    currState().setTwoSidedLighting(on);
    viewer->requestRedraw();
}

//----------------------------------------------------------
void SimResSettings::simresDepthSortButtonPressed()
//----------------------------------------------------------
{
  OsgTransformObject *osg_obj = viewer->getOsgObj();
  OsgDrawableObject *drawable = dynamic_cast<OsgDrawableObject*>(osg_obj);

  if(drawable) {
    osgGA::TrackballManipulator* trackball = dynamic_cast<osgGA::TrackballManipulator*>
      (viewer->getCameraManipulator());
    if (trackball) {
      osg::Matrixd matrix = trackball->getInverseMatrix();
      osg::Vec3 eye, center, up;
      matrix.getLookAt(eye, center, up);
	  
      drawable->sortPrimitives(eye, center);

      //context_->requestRedraw();
      viewer->requestRedraw();
    }
  }
}

//----------------------------------------------------------
void SimResSettings::simresSortModeSelected(const QString& name)
//----------------------------------------------------------
{
  if(name==tr("Re-sort every frame"))
    currState().setSortMode(1);
  else
    currState().setSortMode(0);
  viewer->requestRedraw();

}
    
//----------------------------------------------------------
void SimResSettings::simresBaseColorChanged(QRgb value)
//----------------------------------------------------------
{
    currState().setBaseColor(value);
    viewer->requestRedraw();
}

//----------------------------------------------------------
void SimResSettings::simresBackColorChanged(QRgb value)
//----------------------------------------------------------
{
  currState().setBackColor(value);
  
  //OsgTransformObject *osg_obj = viewer->getOsgObj();
  //OsgDrawableObject *drawable = dynamic_cast<OsgDrawableObject*>(osg_obj);
  //if (drawable)
  //    drawable->setCurrentState(currState().get());
  
  viewer->requestRedraw();
}
    
//----------------------------------------------------------
void SimResSettings::simresWireframeColorChanged(QRgb value)
//----------------------------------------------------------
{
  currState().setWireframeColor(value);

  OsgTransformObject *osg_obj = viewer->getOsgObj();
  osg_obj->updateTransformObject();

  viewer->requestRedraw();
}

//----------------------------------------------------------
void SimResSettings::simresHighlightColorChanged(QRgb value)
//----------------------------------------------------------
{
  currState().setHighlightColor(value);

  OsgTransformObject *osg_obj = viewer->getOsgObj();
  osg_obj->updateTransformObject();

  viewer->requestRedraw();
}
    
//----------------------------------------------------------
void SimResSettings::pointSizeChanged(double v)
//----------------------------------------------------------
{
  currState().setPointSize(v);

  OsgTransformObject *osg_obj = viewer->getOsgObj();
  osg_obj->drawingState().touch();
	
  osg_obj->updateTransformObject();

  viewer->requestRedraw();
}

//----------------------------------------------------------
void SimResSettings::lineWidthChanged(double v)
//----------------------------------------------------------
{
  currState().setLineWidth(v);

  OsgTransformObject *osg_obj = viewer->getOsgObj();
  osg_obj->drawingState().touch();
  osg_obj->updateTransformObject();

  viewer->requestRedraw();
}

//----------------------------------------------------------
void SimResSettings::contoursChanged(bool onoff)
//----------------------------------------------------------
{
  currState().setContour(onoff);

  OsgTransformObject *osg_obj = viewer->getOsgObj();
  osg_obj->updateTransformObject();

  viewer->requestRedraw();
}

//----------------------------------------------------------
void SimResSettings::contourNbChanged(int nb)
//----------------------------------------------------------
{
  currState().setContourNb(nb);

  OsgTransformObject *osg_obj = viewer->getOsgObj();
  osg_obj->updateTransformObject();

  viewer->requestRedraw();
}

//----------------------------------------------------------
void SimResSettings::showBoundingSphereChanged(bool checked)
//----------------------------------------------------------
{
  currState().showBoundingSphere(checked);

  OsgTransformObject *osg_obj = viewer->getOsgObj();
  osg_obj->updateTransformObject();

  viewer->requestRedraw();
}

//----------------------------------------------------------
void SimResSettings::showTriangleNormalsChanged(bool checked)
//----------------------------------------------------------
{
  currState().showTriangleNormals(checked);

  OsgTransformObject *osg_obj = viewer->getOsgObj();
  osg_obj->updateTransformObject();

  viewer->requestRedraw();
}

//----------------------------------------------------------
void SimResSettings::showHighlightChanged(bool checked)
//----------------------------------------------------------
{
  currState().setHighlight(checked);

  OsgTransformObject *osg_obj = viewer->getOsgObj();
  osg_obj->updateTransformObject();

  viewer->requestRedraw();
}

//----------------------------------------------------------
void SimResSettings::showAxesChanged(bool checked)
//----------------------------------------------------------
{
  currState().setShowAxes(checked);

  OsgTransformObject *osg_obj = viewer->getOsgObj();
  osg_obj->updateTransformObject();

  viewer->requestRedraw();
}

//----------------------------------------------------------
void SimResSettings::showPlaneChanged(bool checked)
//----------------------------------------------------------
{
  currState().setShowPlane(checked);

  OsgTransformObject *osg_obj = viewer->getOsgObj();
  osg_obj->updateTransformObject();

  viewer->requestRedraw();
}

//----------------------------------------------------------
SimResState& SimResSettings::currState()
//----------------------------------------------------------
{
  return *viewer->getCurrentState();
}
