#include <gui/FloatSlider.h>
#include <gui/Viewer.h>
#include <gui/TransformSettings.h>
#include <scenegraph/OsgDolfinObject.h>
#include <osg/BoundingBox>
#include <osg/ClipNode>
#include <osgQt/TrackballManipulator.h>
#include <osg/Vec4d>

#include <QLabel>
#include <QPushButton>
#include <QGroupBox>
#include <QCheckBox>

#include <iostream>

//----------------------------------------------------------
TransformSettings::TransformSettings(Viewer* viewer)
//----------------------------------------------------------
  : viewer(viewer)
{
  clip_x_enabled = false;
  clip_y_enabled = false;
  clip_z_enabled = false;

  clip_x_plane = new osg::ClipPlane(0, 1.0, 0.0, 0.0, 0.0);
  clip_y_plane = new osg::ClipPlane(1, 0.0, 1.0, 0.0, 0.0);
  clip_z_plane = new osg::ClipPlane(2, 0.0, 0.0, 1.0, 0.0);
}
//----------------------------------------------------------
TransformSettings::~TransformSettings()
//----------------------------------------------------------
{
}
//----------------------------------------------------------
void TransformSettings::clipPlaneX(bool enable, double x_val)
//----------------------------------------------------------
{
  osg::ClipNode *top = viewer->getClipNode();

  clip_x_enabled = enable;
      
  if (enable) {
    clip_x_plane->setClipPlane(1.0, 0.0, 0.0, x_val);
    top->addClipPlane(clip_x_plane.get());
  } else {
    top->removeClipPlane(clip_x_plane.get());
  }
  viewer->requestRedraw();
}
//----------------------------------------------------------
bool TransformSettings::isClipped_x()
//----------------------------------------------------------
{ 
  return clip_x_enabled; 
}
//----------------------------------------------------------
double TransformSettings::clipValue_x()
//----------------------------------------------------------
{ 
  return clip_x_plane->getClipPlane().w(); 
}
//----------------------------------------------------------
void TransformSettings::clipPlaneY(bool enable, double y_val)
//----------------------------------------------------------
{
  osg::ClipNode *top = viewer->getClipNode();

  clip_y_enabled = enable;
  
  if (enable) {
    clip_y_plane->setClipPlane(0.0, 1.0, 0.0, y_val);
    top->addClipPlane(clip_y_plane.get());
  } else {
    top->removeClipPlane(clip_y_plane.get());
  }
  viewer->requestRedraw();
}
//----------------------------------------------------------
bool TransformSettings::isClipped_y()
//----------------------------------------------------------
{ 
  return clip_y_enabled; 
}
//----------------------------------------------------------
double TransformSettings::clipValue_y()
//----------------------------------------------------------
{ 
  return clip_y_plane->getClipPlane().w(); 
}

//----------------------------------------------------------
void TransformSettings::clipPlaneZ(bool enable, double z_val)
//----------------------------------------------------------
{
  osg::ClipNode *top = viewer->getClipNode();

  clip_z_enabled = enable;
  
  if (enable) {
    clip_z_plane->setClipPlane(0.0, 0.0, 1.0, z_val);
    top->addClipPlane(clip_z_plane.get());
  } else {
    top->removeClipPlane(clip_z_plane.get());
  }
  viewer->requestRedraw();
}

//----------------------------------------------------------
bool TransformSettings::isClipped_z()
//----------------------------------------------------------
{ 
  return clip_z_enabled; 
}
//----------------------------------------------------------
double TransformSettings::clipValue_z()
//----------------------------------------------------------
{ 
  return clip_z_plane->getClipPlane().w(); 
}

//----------------------------------------------------------
void TransformSettings::translateXChanged(double v)
//----------------------------------------------------------
{
  osg::Vec3 trans = viewer->getOsgObj()->getTranslation();
  trans.x() = v;
  viewer->getOsgObj()->setTranslation(trans);
  viewer->requestRedraw();
}
//----------------------------------------------------------
void TransformSettings::translateYChanged(double v)
//----------------------------------------------------------
{
  osg::Vec3 trans = viewer->getOsgObj()->getTranslation();
  trans.y() = v;
  viewer->getOsgObj()->setTranslation(trans);
  viewer->requestRedraw();
}

//----------------------------------------------------------
void TransformSettings::translateZChanged(double v)
//----------------------------------------------------------
{
  osg::Vec3 trans = viewer->getOsgObj()->getTranslation();
  trans.z() = v;
  viewer->getOsgObj()->setTranslation(trans);
  viewer->requestRedraw();
}
//----------------------------------------------------------
void TransformSettings::scaleAllChanged(double v)
//----------------------------------------------------------
{
  viewer->getOsgObj()->setScale(osg::Vec3(v,v,v));
  viewer->requestRedraw();
}

//----------------------------------------------------------
void TransformSettings::scaleXChanged(double v)
//----------------------------------------------------------
{
  osg::Vec3 scale = viewer->getOsgObj()->getScale();
  scale.x() = v;
  viewer->getOsgObj()->setScale(scale);
  viewer->requestRedraw();
}
//----------------------------------------------------------
void TransformSettings::scaleYChanged(double v)
//----------------------------------------------------------
{
  osg::Vec3 scale = viewer->getOsgObj()->getScale();
  scale.y() = v;
  viewer->getOsgObj()->setScale(scale);
  viewer->requestRedraw();
}
//----------------------------------------------------------
void TransformSettings::scaleZChanged(double v)
//----------------------------------------------------------
{
  osg::Vec3 scale = viewer->getOsgObj()->getScale();
  scale.z() = v;
  viewer->getOsgObj()->setScale(scale);
  viewer->requestRedraw();
}
//----------------------------------------------------------
void TransformSettings::rotXChanged(double v)
//----------------------------------------------------------
{
  osg::Vec3 rot_axis = viewer->getOsgObj()->getRotationAxis();
  rot_axis.x() = v;
  viewer->getOsgObj()->setRotationAxis(rot_axis);
  viewer->requestRedraw();
}

//----------------------------------------------------------
void TransformSettings::rotYChanged(double v)
//----------------------------------------------------------
{
  osg::Vec3 rot_axis = viewer->getOsgObj()->getRotationAxis();
  rot_axis.y() = v;
  viewer->getOsgObj()->setRotationAxis(rot_axis);
  viewer->requestRedraw();
}
//----------------------------------------------------------
void TransformSettings::rotZChanged(double v)
//----------------------------------------------------------
{
  osg::Vec3 rot_axis = viewer->getOsgObj()->getRotationAxis();
  rot_axis.z() = v;
  viewer->getOsgObj()->setRotationAxis(rot_axis);
  viewer->requestRedraw();
}
//----------------------------------------------------------
void TransformSettings::rotAngleChanged(double v)
//----------------------------------------------------------
{
  viewer->getOsgObj()->setRotationAngle(M_PI*v/180.0);
  viewer->requestRedraw();
}
//----------------------------------------------------------
void TransformSettings::eulerXChanged(double v)
//----------------------------------------------------------
{
  viewer->getOsgObj()->setEulerXAngle(M_PI*v/180.0);
  viewer->requestRedraw();
}
//----------------------------------------------------------
void TransformSettings::eulerYChanged(double v)
//----------------------------------------------------------
{
  viewer->getOsgObj()->setEulerYAngle(M_PI*v/180.0);
  viewer->requestRedraw();
}
//----------------------------------------------------------
void TransformSettings::eulerZChanged(double v)
//----------------------------------------------------------
{
  viewer->getOsgObj()->setEulerZAngle(M_PI*v/180.0);
  viewer->requestRedraw();
}
//----------------------------------------------------------
void TransformSettings::setAxialView()
//----------------------------------------------------------
{
  viewer->setViewDirection(Viewer::VD_Axial);
  viewer->requestRedraw();
}
//----------------------------------------------------------
void TransformSettings::setCoronaryView()
//----------------------------------------------------------
{
  viewer->setViewDirection(Viewer::VD_Coronary);
  viewer->requestRedraw();
}
//----------------------------------------------------------
void TransformSettings::setSaggitalView()
//----------------------------------------------------------
{
  viewer->setViewDirection(Viewer::VD_Saggital);
  viewer->requestRedraw();
}
