#ifndef Gui_TRANSFORM_SETTINGS_H                // -*- C++ -*-
#define Gui_TRANSFORM_SETTINGS_H

#include <scenegraph/OsgTransformObject.h>
#include <gui/Viewer.h>
#include <osg/ClipPlane>

/**
 *
 * \ingroup gui
 *
 * \author Trond Vidar Stensby
 * \author Benjamin Kehlet
 *
 * \date 2010-03-27
 */

class TransformSettings : public QObject
{
  Q_OBJECT;
  
public:
  TransformSettings(Viewer* viewer);
  ~TransformSettings();

public slots:
  
  // Translation
  void translateXChanged(double v);
  void translateYChanged(double v);
  void translateZChanged(double v);
      
  // Scale
  void scaleAllChanged(double v);
  void scaleXChanged(double v);
  void scaleYChanged(double v);
  void scaleZChanged(double v);

  // Rotation
  void rotXChanged(double v);
  void rotYChanged(double v);
  void rotZChanged(double v);
  void rotAngleChanged(double v);

  /** Uncommented */
  void eulerXChanged(double v);

  /** Uncommented */
  void eulerYChanged(double v);

  /** Uncommented */
  void eulerZChanged(double v);

  void setAxialView();
  void setCoronaryView();
  void setSaggitalView();


  void   clipPlaneX(bool enable, double x_val=0.0);
  bool   isClipped_x();
  double clipValue_x();

  void   clipPlaneY(bool enable, double y_val=0.0);
  bool   isClipped_y();
  double clipValue_y();

  void   clipPlaneZ(bool enable, double z_val=0.0);
  bool   isClipped_z();
  double clipValue_z();

private:
  
  bool clip_x_enabled;
  osg::ref_ptr<osg::ClipPlane> clip_x_plane;

  bool clip_y_enabled;
  osg::ref_ptr<osg::ClipPlane> clip_y_plane;

  bool clip_z_enabled;
  osg::ref_ptr<osg::ClipPlane> clip_z_plane;

  Viewer* viewer;
};

#endif // Gui_TRANSFORM_SETTINGS_H
