#include "TransformationsDialog.h"
#include <gui/ApplicationWindow.h>
#include <gui/FloatSlider.h>
#include <gui/Viewer.h>
#include <gui/TransformSettings.h>

#include <QtGui>

//----------------------------------------------------------
TransformationsDialog::TransformationsDialog(Viewer* viewer, ApplicationWindow* parent)
  : QDialog(parent),
    viewer(viewer)
//----------------------------------------------------------
{

  // Create the tab widget
  QVBoxLayout* layout = new QVBoxLayout(this);
  tabs = new QTabWidget(this);
  tabs->setTabShape(QTabWidget::Rounded);
  tabs->setElideMode(Qt::ElideRight);
  tabs->setUsesScrollButtons(false);

  // Translate page
  translate_page   = new QWidget();
  translate_layout = new QVBoxLayout(translate_page);
  translate_layout->setSpacing(6);
  translate_layout->setMargin(6);

  trans_x_slider = new FloatSlider(translate_page);
  trans_x_slider->setLabel("X:");
  trans_x_slider->setMinValue(-100);
  trans_x_slider->setMaxValue(100);
  trans_x_slider->setGranul(0.01);
  trans_x_slider->setCurrentValue(0);
  translate_layout->addWidget(trans_x_slider);
 
  trans_y_slider = new FloatSlider(translate_page);
  trans_y_slider->setLabel("Y:");
  trans_y_slider->setMinValue(-100);
  trans_y_slider->setMaxValue(100);
  trans_y_slider->setGranul(0.01);
  trans_y_slider->setCurrentValue(0);
  translate_layout->addWidget(trans_y_slider);
  
  trans_z_slider = new FloatSlider(translate_page);
  trans_z_slider->setLabel("Z:");
  trans_z_slider->setMinValue(-100);
  trans_z_slider->setMaxValue(100);
  trans_z_slider->setGranul(0.01);
  trans_z_slider->setCurrentValue(0);
  translate_layout->addWidget(trans_z_slider);

  translate_layout->addItem(new QSpacerItem(2, 2, QSizePolicy::Minimum, QSizePolicy::Expanding));

  tabs->addTab(translate_page, "Translate");
  
  
  // Rotate page
  rotate_page = new QWidget(this);
  rotate_layout = new QVBoxLayout(rotate_page);

  axis_angle_group = new QGroupBox("Axis angle");
  axis_angle_layout = new QVBoxLayout(axis_angle_group);
  axis_angle_layout->setSpacing(6);
  axis_angle_layout->setMargin(6);

  rot_x_slider = new FloatSlider(axis_angle_group);
  rot_x_slider->setLabel("X:");
  rot_x_slider->setMinValue(-1);
  rot_x_slider->setMaxValue(1);
  rot_x_slider->setGranul(0.01);
  rot_x_slider->setCurrentValue(0);
  axis_angle_layout->addWidget(rot_x_slider);
  
  rot_y_slider = new FloatSlider(axis_angle_group);
  rot_y_slider->setLabel("Y:");
  rot_y_slider->setMinValue(-1);
  rot_y_slider->setMaxValue(1);
  rot_y_slider->setGranul(0.01);
  rot_y_slider->setCurrentValue(0);
  axis_angle_layout->addWidget(rot_y_slider);
  
  rot_z_slider = new FloatSlider(axis_angle_group);
  rot_z_slider->setLabel("Z:");
  rot_z_slider->setMinValue(-1);
  rot_z_slider->setMaxValue(1);
  rot_z_slider->setGranul(0.01);
  rot_z_slider->setCurrentValue(1);
  axis_angle_layout->addWidget(rot_z_slider);
  
  rot_angle_slider = new FloatSlider(axis_angle_group);
  rot_angle_slider->setLabel("Angle:");
  rot_angle_slider->setMinValue(-180);
  rot_angle_slider->setMaxValue(180);
  rot_angle_slider->setGranul(0.1);
  rot_angle_slider->setCurrentValue(0);
  axis_angle_layout->addWidget(rot_angle_slider);
  
  rotate_layout->addWidget(axis_angle_group);
  rotate_layout->addItem(new QSpacerItem(2, 2, QSizePolicy::Minimum, QSizePolicy::Expanding));

  tabs->addTab(rotate_page, "Rotate");
  
  /*
  euler_group = new QGroupBox("Euler angles");
  euler_layout = new QVBoxLayout(euler_group);
  euler_layout->setSpacing(6);
  euler_layout->setMargin(6);

  euler_x_slider = new FloatSlider(euler_group);
  euler_x_slider->setMinValue(0);
  euler_x_slider->setMaxValue(1);
  euler_x_slider->setGranul(0.01);
  euler_layout->addWidget(euler_x_slider);
  
  euler_y_slider = new FloatSlider(euler_group);
  euler_y_slider->setMinValue(0);
  euler_y_slider->setMaxValue(1);
  euler_y_slider->setGranul(0.01);
  euler_layout->addWidget(euler_y_slider);
  
  euler_z_slider = new FloatSlider(euler_group);
  euler_z_slider->setMinValue(0);
  euler_z_slider->setMaxValue(1);
  euler_z_slider->setGranul(0.01);
  euler_layout->addWidget(euler_z_slider);

  rotate_layout->addWidget(euler_group);
  */

  // Scale page
  scale_page = new QWidget();
  scale_layout = new QVBoxLayout(scale_page);
  scale_layout->setSpacing(6);
  scale_layout->setMargin(6);

  scale_x_slider = new FloatSlider(scale_page);
  scale_x_slider->setLabel("X:");
  scale_x_slider->setMinValue(0.01);
  scale_x_slider->setMaxValue(100);
  scale_x_slider->setGranul(0.01);
  scale_x_slider->setCurrentValue(1);
  scale_layout->addWidget(scale_x_slider);


  scale_y_slider = new FloatSlider(scale_page);
  scale_y_slider->setLabel("Y:");
  scale_y_slider->setMinValue(0.01);
  scale_y_slider->setMaxValue(100);
  scale_y_slider->setGranul(0.01);
  scale_y_slider->setCurrentValue(1);
  scale_layout->addWidget(scale_y_slider);

  scale_z_slider = new FloatSlider(scale_page);
  scale_z_slider->setLabel("Z:");
  scale_z_slider->setMinValue(0.01);
  scale_z_slider->setMaxValue(100);
  scale_z_slider->setGranul(0.01);
  scale_z_slider->setCurrentValue(1);
  scale_layout->addWidget(scale_z_slider);

  scale_layout->addItem(new QSpacerItem(20, 21, QSizePolicy::Minimum, QSizePolicy::Expanding));
  tabs->addTab(scale_page, "Scale");  


  layout->addWidget(tabs);



  close_button = new QPushButton("Close");
  layout->addWidget(close_button);

  TransformSettings* ts = viewer->transformSettings();

  // Setup connections
  // Translation
  connect(trans_x_slider, SIGNAL(valueChanged(double)),
	  ts, SLOT(translateXChanged(double)));

  connect(trans_y_slider, SIGNAL(valueChanged(double)),
	  ts, SLOT(translateYChanged(double)));

  connect(trans_z_slider, SIGNAL(valueChanged(double)),
	  ts, SLOT(translateZChanged(double)));

  // Rotation
  connect(rot_x_slider, SIGNAL(valueChanged(double)),
	  ts, SLOT(rotXChanged(double)));

  connect(rot_y_slider, SIGNAL(valueChanged(double)),
	  ts, SLOT(rotYChanged(double)));

  connect(rot_z_slider, SIGNAL(valueChanged(double)),
	  ts, SLOT(rotZChanged(double)));

  connect(rot_angle_slider, SIGNAL(valueChanged(double)),
	  ts, SLOT(rotAngleChanged(double)));

  // Scale
  connect(scale_x_slider, SIGNAL(valueChanged(double)),
		   ts, SLOT(scaleXChanged(double)));

  connect(scale_y_slider, SIGNAL(valueChanged(double)),
		   ts, SLOT(scaleYChanged(double)));

  connect(scale_z_slider, SIGNAL(valueChanged(double)),
		   ts, SLOT(scaleZChanged(double)));

  // Close button
  connect(close_button, SIGNAL(clicked()),
	  this, SLOT(accept()));

}
