
#include "Vertex_2D.h"
#include "Scene_2D.h"
#include <QGraphicsScene>
#include <QGraphicsSceneMouseEvent>
#include <QPainter>
#include <QStyle>
#include <QStyleOptionGraphicsItem>
#include <QDebug>


//----------------------------------------------------------
Vertex_2D::Vertex_2D(double x, double y, uint id) 
  : EntityInterface_2D(id), size(.01), p(Qt::gray, .005), b(Qt::gray)
//----------------------------------------------------------
{
  //TODO: Determine size in a clever way

  //qDebug() << "vertex create at (" << x << ", " << y << ")";

  setFlag(ItemSendsGeometryChanges);
  setCacheMode(DeviceCoordinateCache);

  setPos(x,y);
}
//----------------------------------------------------------
void Vertex_2D::setMeshFunctionValue(uint value, QColor& c)
//----------------------------------------------------------
{
  //qDebug() << "Setting value at vertex " << id << " to " << c;

  if (this->value != value)
  {
    // set the value in the mesh
    static_cast<Scene_2D*>(scene())->get_mesh().setVertexBoundaryIndicator(id, value, DolfinObject::SINGLE);

    //set the pen color
    p.setColor(c);
    b.setColor(c);
    
    //schedule a repainting with the new color
    update();
  }
}
//----------------------------------------------------------
void Vertex_2D::updateBoundaryIndicatorColor()
//----------------------------------------------------------
{
  Scene_2D* s =  static_cast<Scene_2D*>(scene());
  uint mesh_value = s->get_mesh().getVertexBoundaryIndicator(id);
  if (mesh_value != value)
  {
    QColor c = s->get_value_color(mesh_value);
    p.setColor(c);
    b.setColor(c);

    update();
  }
}
//----------------------------------------------------------
QRectF Vertex_2D::boundingRect() const
//----------------------------------------------------------
{
  const qreal actual_size = hovered ? size*2 : size;
  QRectF b(-actual_size, -actual_size, actual_size*2, actual_size*2);
  return b;
}
//----------------------------------------------------------
QPainterPath Vertex_2D::shape() const
//----------------------------------------------------------
{
  QPainterPath path;
  const qreal actual_size = hovered ? size*2 : size;
  path.addEllipse(QPointF(.0, .0), actual_size, actual_size);

  return path;
}
//----------------------------------------------------------
void Vertex_2D::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *)
//----------------------------------------------------------
{
   painter->setBrush(b);
   painter->setPen(p);
   const qreal actual_size = hovered ? size * 2 : size;
   painter->drawEllipse(QPointF(0.0, 0.0), actual_size, actual_size);
}
