#include <QDebug>

#include <osgQt/TrackballManipulator.h>

#include <gui/Viewer.h>
#include <gui/Context.h>
#include <gui/ApplicationWindow.h>
#include <gui/DolfinEventHandler.h>

#include <scenegraph/OsgTransformObject.h>
#include <scenegraph/OsgDrawableObject.h>
#include <scenegraph/OsgDolfinObject.h>
#include <osgViewer/ViewerEventHandlers>
#include <osg/ClipNode>
#include <osgGA/GUIEventHandler>
#include <gui/TransformSettings.h>
#include <gui/SimResSettings.h>
#include <gui/MaterialSettings.h>
#include <gui/DrawSettingsDialog.h>
#include <gui/ViewerSettingsDialog.h>
#include <gui/ClipPlanesDialog.h>
#include <gui/TransformationsDialog.h>
#include <gui/VisualizationDialog.h>


#include <iostream>

//#define DEBUG_ADD_OBJECT

static const double _box_margin_ =  0.1;

//----------------------------------------------------------
Viewer::Viewer(Context& context, DolfinObject& mesh, QWidget *parent)
  : mesh(mesh),
    RenderArea(parent),
    context_(context),
    transform_settings(new TransformSettings(this)),
    simres_settings(new SimResSettings(this)),
    material_settings(new MaterialSettings(this))
//----------------------------------------------------------
{
  qDebug() << "Viewer::constructor";

  //TODO: It is error prone to have to do this outside the Osg classes
  // Is it possible that this can be done automatically when creating
  // an OsgDolfin object?
  initSceneGraph();
  qDebug() << "Scenegraph initalized";


  initViewerMenu();

  // Load the new mesh
  OsgDolfinObject* osgDolfin_object = new OsgDolfinObject(mesh);

  // TODO: Can't this be done in the constructor of DolfinObject
  osgDolfin_object->initialize();
  osgDolfin_object->updateMatrix();

  addOsgObject(osgDolfin_object);

  setTrackballCenter(osgobj_);
}

//----------------------------------------------------------
Viewer::~Viewer()
//----------------------------------------------------------
{
}


//----------------------------------------------------------
osg::Group *Viewer::getParentGroup(OsgTransformObject *obj)
//----------------------------------------------------------
{
  osg::Group *root = root_node_.get();
  return root;
}
//----------------------------------------------------------
void Viewer::setName(const QString &name)
//----------------------------------------------------------
{
  setObjectName(name);
}

//----------------------------------------------------------
QString Viewer::getName() const
//----------------------------------------------------------
{
  return objectName();
}

//----------------------------------------------------------
void Viewer::initSceneGraph()
//----------------------------------------------------------
{
  root_node_ = new osg::ClearNode();
  root_node_->setClearColor(osg::Vec4(65.0/255.0, 65.0/255.0, 98.0/255, 1.0));
  
  osg::StateSet *state = root_node_->getOrCreateStateSet();
  state->setMode(GL_LIGHTING, osg::StateAttribute::ON);
  
  setRedrawPolicy(AS_NEEDED);
  //setRedrawPolicy(CONSTANT);

  clip_node_ = new osg::ClipNode();
  clip_node_->addChild(root_node_.get());
  clip_node_->setName("Top node");

  ::TrackballManipulator* manip = new ::TrackballManipulator();
  osgViewer::StatsHandler* stats_handler = new osgViewer::StatsHandler();

  setSceneData(clip_node_.get());
  setCameraManipulator(manip);
  addEventHandler(stats_handler);
}
//----------------------------------------------------------
void Viewer::addOsgObject(OsgDolfinObject *obj)
//----------------------------------------------------------
{
  // Check category to find parent osg group (default is root node)
  osg::Group *root = getParentGroup(obj);
      
  if (!root->containsNode(obj)) {
    root->addChild(obj);
  }

  osgobj_ = obj;

  //OsgDrawable* osg_drawable = dynamic_cast<OsgDrawable*> osgobj_;
  //connect(
}



//----------------------------------------------------------
osg::Group *Viewer::getRootNode()
//----------------------------------------------------------
{
  return root_node_.get();
}

//----------------------------------------------------------
void Viewer::setBackgroundColor(const QColor& color)
//----------------------------------------------------------
{
  osg::Vec4 vec(color.red()/255.0,
		color.green()/255.0,
		color.blue()/255.0,
		color.alpha()/255.0);
  
  root_node_->setClearColor(vec);
  updateGL();
}

//----------------------------------------------------------
const QColor Viewer::getBackgroundColor() const
//----------------------------------------------------------
{
  const osg::Vec4& col = root_node_->getClearColor();
  return QColor(255*col[0],255*col[1], 255*col[2],255*col[3]);
}

//----------------------------------------------------------
void Viewer::setViewDirection(const ViewDirection vd)
//----------------------------------------------------------
{
  // If there is no object return here. Othervise the app crashes!
  if(!clip_node_.get())
    return;
  
  ::TrackballManipulator* trackball = getTrackballManipulator();

  if (!trackball) 
    return;

  osg::Vec3 eye, center, up;
  
  //const osg::BoundingSphere& bs = trackball->getNode()->getBound();
  osg::BoundingSphere  bs;
  if (!getOsgObj()) {
    bs = clip_node_->getBound();
  } else {
    OsgTransformObject *osg_obj = getOsgObj();
    bs = osg_obj->getBound();	
  }
  
  center = bs.center();
  
  osg::Matrixd matrix = trackball->getInverseMatrix();
  
  ::TrackballManipulator::State state = trackball->getTrackballState();

  const float d = 3.5f * bs.radius();
  
  // Set eye, center and up
  switch (vd) {
  case VD_Axial:
    up.set(0,1,0);
    eye = bs.center() + osg::Vec3( 0.0f, 0.0f, d);
    break;
  case VD_Coronary:
    up.set(0,0,1);
    eye = bs.center() + osg::Vec3( 0.0, -d, 0.0f);
    break;
  case VD_Saggital:
    up.set(0,0,1);
    eye = bs.center() + osg::Vec3( d, 0.0, 0.0f);
    break;
  default:
    up.set(0,1,0);
  }
  
  matrix.makeLookAt(eye, center, up);
  
  state.matrix = matrix;
  
  trackball->setTrackballState(state);
  trackball->makeLookAt(eye, center, up);
  //trackball->setByMatrix(matrix);
  
  requestRedraw();
}
//----------------------------------------------------------
OsgDolfinObject* Viewer::getOsgObj() 
//----------------------------------------------------------
{
  return osgobj_;
}

//----------------------------------------------------------
osg::ClipNode* Viewer::getClipNode() 
//----------------------------------------------------------
{
  return clip_node_;
}
//----------------------------------------------------------
SimResSettings* Viewer::getSimres_settings()
//----------------------------------------------------------
{ 
  //simres_settings->setActiveSimResState(getOsgObj()->getState());
  return simres_settings; 
}
//----------------------------------------------------------
MaterialSettings* Viewer::getMaterial_settings()
//----------------------------------------------------------
{ 
  //simres_settings->setActiveSimResState(getOsgObj()->getState());
  return material_settings; 

}
//----------------------------------------------------------
SimResState* Viewer::getCurrentState()
//----------------------------------------------------------
{ 
  //OsgDrawableObject* osgDraw = dynamic_cast<OsgDrawableObject*>(getOsgObj());
  return &getOsgObj()->getCurrentState();
}
//----------------------------------------------------------
void Viewer::removeEventHandler(osgGA::GUIEventHandler* event_handler)
//----------------------------------------------------------
{
  //Traverse list of event handlers and remove event handler if found
  
  //EventHandlers is a typedef of std::list<osgGA::GUIEventHandlers>
  osgViewer::Viewer::EventHandlers& handlers = getEventHandlers();

  handlers.remove(event_handler);

  /*  
  for (EventHandlers::iterator it = handlers.begin(); it != handlers.end(); it++)
  { 
    osgGA::GUIEventHandler* current = &(*it);
    if 
  }
  return false;
  */
}
//----------------------------------------------------------
void Viewer::initViewerMenu()
//----------------------------------------------------------
{ 
  QAction* trnsf_action = new QAction(QString("Transformations"), this);
  connect(trnsf_action, SIGNAL(triggered()),
          this, SLOT(transformations()));
  viewer_menu.append(trnsf_action);

  QAction* vis_action = new QAction(QString("Visualization"), this);
  connect(vis_action, SIGNAL(triggered()),
          this, SLOT(visualization()));
  viewer_menu.append(vis_action);

  QAction* draw_action = new QAction(QString("Draw settings"), this);
  connect(draw_action, SIGNAL(triggered()),
          this, SLOT(drawSettings()));
  viewer_menu.append(draw_action);

  QAction *clipplanes_action = new QAction(QString("Clip planes"), this);
  connect(clipplanes_action, SIGNAL(triggered()),
          this, SLOT(clipPlanes()));
  viewer_menu.append(clipplanes_action);
}
//----------------------------------------------------------
void Viewer::enableMeshFunctions(bool enable)
//----------------------------------------------------------
{
  if (enable) 
  {
    // Install the meshfunction event handler
    current_event_handler = new DolfinEventHandler(context_.app()->meshfunctions_toolbar);
    current_event_handler->setViewer(this);
    
    addEventHandler(current_event_handler);

  } 
  else if (current_event_handler != NULL) 
  {
    std::cerr << "Disable event handler" << std::endl;
      
    //Remove the event handler
    removeEventHandler(current_event_handler);
  }
}
//----------------------------------------------------------
void Viewer::updateBoundaryIndicatorColors()
//----------------------------------------------------------
{
  osgobj_->updateBoundaryIndicatorColors();
}
//----------------------------------------------------------
void Viewer::viewCenter()
//----------------------------------------------------------
{
  setTrackballCenter(osgobj_);
  requestRedraw();
}
//----------------------------------------------------------
void Viewer::drawSettings() 
//----------------------------------------------------------
{
  DrawSettingsDialog d(this);
  d.exec();
}
//----------------------------------------------------------
void Viewer::viewerSettings() 
//----------------------------------------------------------
{
  ViewerSettingsDialog d(this);
  d.exec();
}
//----------------------------------------------------------
void Viewer::clipPlanes() 
//----------------------------------------------------------
{
  ClipPlanesDialog d(this);
  d.exec();
}
//----------------------------------------------------------
void Viewer::transformations() 
//----------------------------------------------------------
{
  TransformationsDialog d(this);
  d.exec();
}
//----------------------------------------------------------
void Viewer::visualization() 
//----------------------------------------------------------
{
  VisualizationDialog d(this, context_.app());
  d.exec();
}
//----------------------------------------------------------
void Viewer::setVertexBoundaryIndicator(uint id, int value)
//----------------------------------------------------------
{
  osgobj_->setVertexBoundaryIndicator(id, value);
}
//----------------------------------------------------------
void Viewer::setEdgeBoundaryIndicator(uint id, int value)
//----------------------------------------------------------
{
  osgobj_->setEdgeBoundaryIndicator(id, value);
}
//----------------------------------------------------------
void Viewer::setFaceBoundaryIndicator(uint id, int value)
//----------------------------------------------------------
{
  osgobj_->setFaceBoundaryIndicator(id, value);
}

void Viewer::saveImage(QString filename){}
void Viewer::undo(){}
