#ifndef Gui_Viewer_h_Included             // -*- C++ -*-
#define Gui_Viewer_h_Included   

#include <osgQt/RenderArea.h>
#include <util/SimResState.h>
#include <gui/Context.h>
#include <gui/DolfinObject.h>
#include <gui/ViewerInterface.h>

#include <osg/Group>
#include <osgViewer/Viewer>

#include <QObject>
#include <QColor>

class QDialog;
class QGraphicsView;
class QGraphicsProxyWidget;
class QGLWidget;
class QMenuBar;
class OsgDolfinObject;

namespace osg{ class ClipNode; } 
namespace osgGA{ class GUIEventHandler; }

class DolfinEventHandler;
class Context;

/// Viewer -- QT viewer.
/**
 * QT viewer. Based on Osg example program.
 *
 * \author Benjamin Kehlet (modifications)
 * \date 2011-01-28
 *
 */

class Viewer : public RenderArea, public ViewerInterface
{
  Q_OBJECT;
  
 public:
      
  enum ViewDirection 
  {
    VD_Saggital,
    VD_Coronary,
    VD_Axial
  };

  /** Creation */
  Viewer(Context& context, DolfinObject& mesh, QWidget *parent = 0);
  
  /** Desctruction */
  virtual ~Viewer();
  
  /** Returns global application context */
  //Context *context();

  /** Sets viewer name */
  void setName(const QString &name);
  
  /** Returns viewer name */
  QString getName() const;

  // TODO: Move to osgDolfinObject
  /** Initialize scenegraph */
  virtual void initSceneGraph();

  /** Add an object to the local scenegraph of this viewer */
  virtual void addOsgObject(OsgDolfinObject *obj);
      
  
  /** Returns background color  */
  const QColor getBackgroundColor() const;
  
  /** Get root node */
  osg::Group *getRootNode();
  
  /** Reset view direction */
  void setViewDirection(const ViewDirection vd);
  
  OsgDolfinObject* getOsgObj();
  osg::ClipNode* getClipNode();
  TransformSettings* transformSettings(){ return transform_settings; }

  SimResState*       getCurrentState();
  SimResSettings*    getSimres_settings();
  MaterialSettings*  getMaterial_settings();					 
  
  /** Convenience function that traverses the list of event 
   * handler and removes
   */
  void removeEventHandler(osgGA::GUIEventHandler* event_handler);
                                                                
  /** ViewerInterface implementation */
  void enableMeshFunctions(bool enable);
  void viewCenter();

  void setVertexBoundaryIndicator(uint id, int value);
  void setEdgeBoundaryIndicator(uint id, int value);
  void setFaceBoundaryIndicator(uint id, int value);
  void updateBoundaryIndicatorColors();

  /** Not implemented */
  void saveImage(QString filename);

  /** Not implemented */
  void undo();

  QList<QAction*> getViewerMenu() { return viewer_menu; }
  QWidget* get_widget() { return this; }

  DolfinObject& get_mesh() { return mesh; }

public slots:
  /** Set background color  */
  void setBackgroundColor(const QColor& color);


   /** View the transformations dialog. Mostly taken from old ToolboxDock */
   virtual void transformations();

   /** View the visualization dialog. Mostly taken from old ToolboxDock */
   virtual void visualization();

   /** View the viewer settings dialog. Taken from the old ToolboxDock */
   virtual void viewerSettings();
   /** View the clip planes dialog (which also includes setting view direction. 
    *  Taken from old ToolboxDock */

   /** View the draw settings dialog. Moved from old meshfunctions tab in toolbox */
   virtual void drawSettings();

  /** View the clip planes dialog (which also includes setting view direction. */
  virtual void clipPlanes();

protected:

  /** Find group in with object belong */
  osg::Group *getParentGroup(OsgTransformObject *obj);
  
  /** Base class */
  Viewer *parent_;
  
  /** Global application context */
  Context& context_;
  
  /** Top root node */
  osg::ref_ptr<osg::ClearNode> root_node_;

  /** Top clipping node */
  osg::ref_ptr<osg::ClipNode> clip_node_;  

private:
  DolfinObject& mesh;

  DolfinEventHandler* current_event_handler;

  OsgDolfinObject*   osgobj_;
  TransformSettings* transform_settings;
  SimResSettings*    simres_settings;
  MaterialSettings*  material_settings;

  QList<QAction*> viewer_menu;
  void initViewerMenu();
};


#endif // Gui_Viewer_h_Included
