#ifndef VIEWER_SETTINGS_DIALOG
#define VIEWER_SETTINGS_DIALOG

#include <QDialog>

class ApplicationWindow;
class ColorButton;
class QVBoxLayout;
class QGroupBox;
class QComboBox;
class QSpacerItem;
class Viewer;

/** Defines the dialog viewer setting. 
 *  Let the user set background color, and 
 *  ambient, specular and diffuse light colors.
 *  Most of it is taken from the "global settings"
 *  tab of the Toolbox in earlier versions of Meshbuilder.
 *
 *  \author Benjamin Kehlet
 *  \since 2010-03-26
 *  \date  2010-03-27
 */
class ViewerSettingsDialog : public QDialog {
  Q_OBJECT

public:
  ViewerSettingsDialog(Viewer* viewer, ApplicationWindow* parent=0);


 protected slots:
  void changeWindowSize(const QString& arg);
  void changeLightAmbient(const QColor& c);
  void changeLightDiffuse(const QColor& c);
  void changeLightSpecular(const QColor& c);

 private:
  QVBoxLayout *verticalLayout;
  ColorButton *primary_background_color;
  ColorButton *ambient_light_color;
  ColorButton *diffuse_light_color;
  ColorButton *specular_light_color;
  QComboBox   *window_size_combo;
  QSpacerItem *spacer;
  QPushButton *closeButton;
  Viewer      *viewer;
    
};

#endif
