#ifndef Gui_Viewer2D_h_Included             // -*- C++ -*-
#define Gui_Viewer2D_h_Included   

#include <QGraphicsView>
#include <gui/Context.h>
#include <gui/ViewerInterface.h>
#include <dolfin/mesh/Mesh.h>


namespace dolfin {
  class Mesh;
}

class QGraphicsScene;
class QWheelEvent;
class Scene_2D;
class Axes_2D;

/**
 *  A widget for drawing 2D dolfin meshes. Utilizes the Qt GraphicsScene 2D painter
 *  for rendering.
 *
 * \author Benjamin Kehlet
 * \date  2011-02-20
 * \since 2010-03-16
 */

class Viewer_2D : public QGraphicsView, public ViewerInterface 
{
  Q_OBJECT;

 public :
  Viewer_2D(Context& context, DolfinObject& mesh, QWidget* parent=0);
  virtual ~Viewer_2D();


  // Implement ViewerInterface
  void enableMeshFunctions(bool enable);
  void viewCenter();

  void setVertexBoundaryIndicator(uint id, int value);
  void setEdgeBoundaryIndicator(uint id, int value);
  void setFaceBoundaryIndicator(uint id, int value);
  void updateBoundaryIndicatorColors();

  /** Not implemented */
  void saveImage(QString filename);

  /** Not implemented */
  void undo();

  QWidget* get_widget() { return this; }
  QList<QAction*> getViewerMenu() { return viewer_menu; }
  DolfinObject& get_mesh() { return mesh; }
  

public slots :
  void showAxesChanged(bool show);

protected :
  void wheelEvent(QWheelEvent *event);

protected slots :
  void visualization();

private:
  Context& context;
  Scene_2D* scene;
  DolfinObject  &mesh;

  Axes_2D* axes;
  bool showAxes;

  QList<QAction*> viewer_menu;
  void initViewerMenu();
};

#endif
