#ifndef VISUALIZATION_DIALOG_H
#define VISUALIZATION_DIALOG_H

#include <QDialog>

class ApplicationWindow;
class QTabWidget;
class QWidget;
class QVBoxLayout;
class QHBoxLayout;
class QCheckBox;
class QGroupBox;
class QFormLayout;
class QRadioButton;
class QPushButton;
class QComboBox;
class QSpinBox;
class QButtonGroup;
class ColorButton;
class FloatSlider;
class SimResSettings;
class MaterialSettings;
class Viewer;

/** Defines the dialog for visulization settings. 
 *  Mostly taken from visualization tab in old toolbox.
 *
 *  \author Benjamin Kehlet
 *  \since 2010-03-28
 *  \date  2010-03-29
 */

class VisualizationDialog : public QDialog {
  Q_OBJECT

public:
  VisualizationDialog(Viewer* viewer, ApplicationWindow* parent);

 private:
  QTabWidget*  tabs;

  // Coloring tab
  QWidget*      coloring_page;
  QVBoxLayout*  coloring_page_layout;
  QGroupBox*    render_mode_groupbox;

  /** The is the collection of the radiobutton. It provides
   *  the signal buttonClicked() */
  QButtonGroup* render_mode;
  QFormLayout*  form_layout;
  QRadioButton* smooth_button;
  QRadioButton* wireframe_button;
  QRadioButton* flat_button;
  QRadioButton* points_button;
  QRadioButton* wire_and_flat_button;
  QRadioButton* wire_and_smooth_button;

  ColorButton*  base_color; 
  ColorButton*  back_color;
  ColorButton*  highlight_color;
  ColorButton*  wireframe_color; 
  
  QCheckBox*    show_highlight;
  QCheckBox*    show_axes;
  QCheckBox*    show_plane;

  // Material tab
  QWidget*     material_page;
  QVBoxLayout* material_layout;
  
  FloatSlider* transparency_slider;
  FloatSlider* ambient_slider;  
  FloatSlider* diffuse_slider;
  FloatSlider* emission_slider;
  FloatSlider* specular_slider;
  FloatSlider* shininess_slider;

  //Advanced tab
  QWidget*      advanced_page;
  QVBoxLayout*  advanced_layout;
  QGroupBox*    transparent_group;
  QHBoxLayout*  transparent_layout;
  QComboBox*    sort_mode_combo;
  QPushButton*  depth_sort_button;

  QGroupBox*    culling_panel;
  QButtonGroup* culling_button_group;
  QHBoxLayout*  culling_layout;
  QRadioButton* cull_mode_off;
  QRadioButton* cull_mode_back;
  QRadioButton* cull_mode_front;

  QGroupBox*    light_model_panel;
  QVBoxLayout*  light_model_layout;
  QCheckBox*    two_sided_checkbox;

  QGroupBox*    points_group;
  QVBoxLayout*  points_layout;
  FloatSlider*  point_size;
  
  QGroupBox*    line_group;
  QVBoxLayout*  line_layout;
  FloatSlider*  line_width;
  
  QGroupBox*    contours;
  QHBoxLayout*  contour_layout;
  QSpinBox*     contour_nb;

  QGroupBox*    extras_group;
  QVBoxLayout*  extras_layout;
  QCheckBox*    show_bounding_sphere;
  QCheckBox*    show_triangle_normals;
  
  

  QPushButton*  close_button;

  SimResSettings*   simres_settings;
  MaterialSettings* material_settings;
};

#endif
