#include <osgQt/AdapterWidget.h>

#include <QGraphicsSceneMouseEvent>
#include <QGraphicsScene>

#include <iostream>

using namespace std;

//#define DEBUG_EVENT

//----------------------------------------------------------
AdapterWidget::AdapterWidget(QWidget * parent, const char *, 
			     const QGLWidget * shareWidget, WindowFlags f)
  : QGLWidget(QGLFormat(QGL::SampleBuffers), parent, shareWidget, f)
//----------------------------------------------------------
{
  //NOTE: Name not used at the moment? Should it?


  _gw = new osgViewer::GraphicsWindowEmbedded(0,0,width(),height());
  //setMouseTracking(true);
  
  //setFocusPolicy(Qt::ClickFocus);
  ////setFocusPolicy(Qt::StrongFocus);
  setFocusPolicy(Qt::StrongFocus);
}

//----------------------------------------------------------
AdapterWidget::~AdapterWidget()
//----------------------------------------------------------
{
}

//----------------------------------------------------------
void AdapterWidget::resizeGL( int width, int height )
//----------------------------------------------------------
{
  _gw->getEventQueue()->windowResize(0, 0, width, height );
  _gw->resized(0,0,width,height);
  //debugCurrentEventState();
}

//----------------------------------------------------------
void AdapterWidget::keyPressEvent( QKeyEvent* event )
//----------------------------------------------------------
{
  //cerr << "AdapterWidget::keyPressEvent" << endl;

  int osg_key=0, qt_ascii;

  qt_ascii = *(event->text().toAscii().data());

  int qt_key = event->key();
  switch(qt_key)
  {
    case Qt::Key_Control:
      osg_key = osgGA::GUIEventAdapter::KEY_Control_L;
      break;
    default:
      osg_key = qt_ascii;
  }
   
  if(osg_key > 0)
  {
    _gw->getEventQueue()->keyPress(osg_key);
    //debugCurrentEventState();
  }
  processAdaptedEvents();
}

//----------------------------------------------------------
void AdapterWidget::keyReleaseEvent( QKeyEvent* event )
//----------------------------------------------------------
{
  //cerr << "AdapterWidget::keyReleaseEvent" << endl;
  int osg_key=0, qt_ascii;

  qt_ascii = *(event->text().toAscii().data());
    
  int qt_key = event->key();
  switch(qt_key)
  {
    case Qt::Key_Control:
      osg_key = osgGA::GUIEventAdapter::KEY_Control_L;
      break;
    default:
      osg_key = qt_ascii;
  }

  if(osg_key > 0)
  {
      _gw->getEventQueue()->keyRelease(osg_key);
      //debugCurrentEventState();
  }
  processAdaptedEvents();
}

//----------------------------------------------------------
void AdapterWidget::mousePressEvent( QMouseEvent* event )
//----------------------------------------------------------
{
  int button = getButtonAndProcessModifiers(event);
#ifdef DEBUG_EVENT
  cerr << "AdapterWidget::mousePressEvent" << endl;
  cerr << "\tbutton=" << button << ", x=" << event->x() << ", y=" << event->y() << endl;
#endif
  _gw->getEventQueue()->mouseButtonPress(event->x(), event->y(), button);
  //debugCurrentEventState();
  processAdaptedEvents();
}

//----------------------------------------------------------
void AdapterWidget::mouseReleaseEvent( QMouseEvent* event )
//----------------------------------------------------------
{
  int button = getButtonAndProcessModifiers(event);
#ifdef DEBUG_EVENT
  cerr << "AdapterWidget::mouseReleaseEvent" << endl;
  cerr << "\tbutton=" << button << ", x=" << event->x() << ", y=" << event->y() << endl;
#endif
  _gw->getEventQueue()->mouseButtonRelease(event->x(), event->y(), button);
  //debugCurrentEventState();
  processAdaptedEvents();
}

//----------------------------------------------------------
void AdapterWidget::mouseMoveEvent( QMouseEvent* event )
//----------------------------------------------------------
{
  //int button = getButtonAndProcessModifiers(event);
#ifdef DEBUG_EVENT
  cerr << "AdapterWidget::mouseMoveEvent" << endl;
  cerr << "\tx=" << event->x() << ", y=" << event->y() << endl;
#endif
  _gw->getEventQueue()->mouseMotion(event->x(), event->y());
  //debugCurrentEventState();
  processAdaptedEvents(); //updateGL();
}

//----------------------------------------------------------
void AdapterWidget::wheelEvent( QWheelEvent* event )
//----------------------------------------------------------
{
  //cerr << "AdapterWidget::wheelEvent" << endl;
  if(event->delta() > 0)
    _gw->getEventQueue()->mouseScroll(osgGA::GUIEventAdapter::SCROLL_UP);
  else
    _gw->getEventQueue()->mouseScroll(osgGA::GUIEventAdapter::SCROLL_DOWN);
  //debugCurrentEventState();
  processAdaptedEvents(); //updateGL();
}

//----------------------------------------------------------
int AdapterWidget::getButtonAndProcessModifiers(QMouseEvent* event) 
//----------------------------------------------------------
{
  int button = 0;
  switch(event->button())
    {
    case(Qt::LeftButton): button = 1; break;
    case(Qt::MidButton): button = 2; break;
    case(Qt::RightButton): button = 3; break;
    case(Qt::NoButton): button = 0; break;
    default: button = 0; break;
    }

  osgGA::GUIEventAdapter* ea=_gw->getEventQueue()->getCurrentEventState();

  if(event->modifiers() & Qt::ControlModifier)
    ea->setModKeyMask(osgGA::GUIEventAdapter::MODKEY_CTRL | ea->getModKeyMask());
  else
    ea->setModKeyMask(~osgGA::GUIEventAdapter::MODKEY_CTRL & ea->getModKeyMask());

  if(event->modifiers() & Qt::ShiftModifier)
    ea->setModKeyMask(osgGA::GUIEventAdapter::MODKEY_SHIFT | ea->getModKeyMask());
  else
    ea->setModKeyMask(~osgGA::GUIEventAdapter::MODKEY_SHIFT & ea->getModKeyMask());

  if(event->modifiers() & Qt::AltModifier)
    ea->setModKeyMask(osgGA::GUIEventAdapter::MODKEY_ALT | ea->getModKeyMask());
  else
    ea->setModKeyMask(~osgGA::GUIEventAdapter::MODKEY_ALT & ea->getModKeyMask());

  if(event->modifiers() & Qt::MetaModifier)
    ea->setModKeyMask(osgGA::GUIEventAdapter::MODKEY_META | ea->getModKeyMask());
  else
    ea->setModKeyMask(~osgGA::GUIEventAdapter::MODKEY_META & ea->getModKeyMask());

  return button;
}

//----------------------------------------------------------
void AdapterWidget::debugCurrentEventState()
//----------------------------------------------------------
{
  osgGA::GUIEventAdapter* ea = _gw->getEventQueue()->getCurrentEventState();
  if(ea==NULL)
  {
    cerr << "CurrentEventState: undefined" << endl;
    return;
  }

  cerr << "CurrentEventState: " << endl;
  cerr << "\tEventType=" << ea->getEventType() << endl;
  cerr << "\tWindowX=" << ea->getWindowX() << endl;
  cerr << "\tWindowY=" << ea->getWindowY() << endl;
  cerr << "\tWindowWidth=" << ea->getWindowWidth() << endl;
  cerr << "\tWindowHeight=" << ea->getWindowHeight() << endl;
  cerr << "\tKey=" << ea->getKey() << endl;
  cerr << "\tButton=" << ea->getButton() << endl;
  cerr << "\tXmin=" << ea->getXmin() << endl;
  cerr << "\tXmax=" << ea->getXmax() << endl;
  cerr << "\tYmin=" << ea->getYmin() << endl;
  cerr << "\tYmax=" << ea->getYmax() << endl;
  cerr << "\tX=" << ea->getX() << endl;
  cerr << "\tY=" << ea->getY() << endl;
  cerr << "\tButtonMask=" << ea->getButtonMask() << endl;
  cerr << "\tModKeyMask=" << ea->getModKeyMask() << endl;
  cerr << "\tScrollingDeltaX=" << ea->getScrollingDeltaX() << endl;
  cerr << "\tScrollingDeltaY=" << ea->getScrollingDeltaY() << endl;
  cerr << "\tScrollingMotion=" << ea->getScrollingMotion() << endl;
  cerr << "\tXnormalized=" << ea->getXnormalized() << endl;
  cerr << "\tYnormalized=" << ea->getYnormalized() << endl;
  cerr << "\tMouseYOrientation=" << ea->getMouseYOrientation() << endl;
}

