#include <osgQt/TrackballManipulator.h>
#include <iostream>

//----------------------------------------------------------
TrackballManipulator::TrackballManipulator()
//----------------------------------------------------------
{
  setMinimumDistance(0.05);
}

//----------------------------------------------------------
TrackballManipulator::~TrackballManipulator()
//----------------------------------------------------------
{
  
}

//----------------------------------------------------------
TrackballManipulator::State TrackballManipulator::getTrackballState() const
//----------------------------------------------------------
{
  State s;
  s.matrix = getMatrix();
  s.center = _center;
  s.rotation = _rotation;
  s.modelScale = _modelScale;
  s.minimumZoomScale = _minimumZoomScale;
  s.distance = _distance;
  
  return s;
}

//----------------------------------------------------------
void TrackballManipulator::setTrackballState(State s)
//----------------------------------------------------------
{
  setByMatrix(s.matrix);
  _center = s.center;
  _rotation = s.rotation;
  _modelScale = s.modelScale;
  _minimumZoomScale = s.minimumZoomScale;
  _distance = s.distance;
}

//----------------------------------------------------------
void TrackballManipulator::setCenter(osg::Vec3 center)
//----------------------------------------------------------
{
  osg::Vec3 eye, tmp, up;
  getInverseMatrix().getLookAt(eye, tmp, up);
  
  computePosition(eye, center, up);
}
  
//----------------------------------------------------------
void TrackballManipulator::makeLookAt(osg::Vec3 eye, osg::Vec3 center, osg::Vec3 up)
//----------------------------------------------------------
{
  computePosition(eye, center, up);
}
  
//----------------------------------------------------------
void TrackballManipulator::setNode(osg::Node* node)
//----------------------------------------------------------
{
  osgGA::TrackballManipulator::setNode(node);
    
  if(_modelScale < 0.0)
    _modelScale = -_modelScale;
}
    
//----------------------------------------------------------
void TrackballManipulator::debugPrint() const
//----------------------------------------------------------
{
  std::cerr << "\ttb center: " <<  _center.x() << " " << _center.y() << " " <<  _center.z() << std::endl
	    << "\ttb rotation: " << _rotation.x() << " " <<  _rotation.y() << " " << _rotation.z()
	    << " " << _rotation.w() << std::endl
	    << "\ttb modelScale: " << _modelScale << std::endl
	    << "\ttb minimumZoomScale: " << _minimumZoomScale << std::endl
	    << "\ttb distance: "<<  _distance << std::endl;
}
  
void TrackballManipulator::computePosition(const osg::Vec3& eye,const osg::Vec3& center,const osg::Vec3& up)
{

    osg::Vec3 lv(center-eye);

    osg::Vec3 f(lv);
    f.normalize();
    osg::Vec3 s(f^up);
    s.normalize();
    osg::Vec3 u(s^f);
    u.normalize();
    
    osg::Matrix rotation_matrix(s[0],     u[0],     -f[0],     0.0f,
                                s[1],     u[1],     -f[1],     0.0f,
                                s[2],     u[2],     -f[2],     0.0f,
                                0.0f,     0.0f,     0.0f,      1.0f);
                   
    _center = center;
    _distance = lv.length();
    _rotation = rotation_matrix.getRotate().inverse();
}

