#ifndef OsgQt_TRACKBALL_MANIPULATOR_H               // -*- C++ -*-
#define OsgQt_TRACKBALL_MANIPULATOR_H

#include <osgGA/TrackballManipulator>

class TrackballManipulator : public osgGA::TrackballManipulator
{
public:
  TrackballManipulator();
  virtual ~TrackballManipulator();
  
  /// Struct for saving the state of the manipulator.
  struct State 
  {
    osg::Matrixd matrix;
    osg::Vec3 center;
    osg::Quat rotation;
    double modelScale;
    double minimumZoomScale;
    double distance;
  };
  
  /// Get the current center for the trackball manipulator
  State getTrackballState() const;
  
  /// Set the current center for the trackball manipulator
  void setTrackballState(State s);
  
  /// Set a new center for this trackball
  void setCenter(osg::Vec3 center);
  
  void makeLookAt(osg::Vec3 eye, osg::Vec3 center, osg::Vec3 up);
  
  /// Bugfix. Error in parent implementaton?
  virtual void setNode(osg::Node* node);
  
  // Print current state
  void debugPrint() const;

  void computePosition(const osg::Vec3& eye,const osg::Vec3& lv,const osg::Vec3& up);

protected:
	double _modelScale;
	double _minimumZoomScale;
};

#endif // OsgQt_TRACKBALL_MANIPULATOR_H
