#ifndef Osg_OSGDOLFINOBJECT_H             // -*- C++ -*-
#define Osg_OSGDOLFINOBJECT_H

#include <scenegraph/OsgMeshObject.h>
#include <gui/DolfinObject.h>

/**
 * Contains Dolfin mesh data and properties for rendering them
 *
 * \ingroup scenegraph
 *
 * \author Ola Røer Thorsen
 * \author Benjamin Kehlet
 * \since 2007-06-18
 * \date  2011-01-30
 */
class OsgDolfinObject : public OsgMeshObject
{
 public:
  
  OsgDolfinObject(DolfinObject& dolfin_object);
  OsgDolfinObject(const OsgDolfinObject& obj);
  
  virtual ~OsgDolfinObject();
  
  virtual void initialize();

  // Vertex boundary indicator functions
  void setVertexBoundaryIndicator(uint id, int value);
  void fillVertexBoundaryIndicator(uint id, int value);
  //void fillVertexSelectedIndicator(uint id, int value);
  //void setVertexAllBoundaryIndicators(int value);


  // Edge boundary indicator functions
  uint findEdgeFromVertices(uint id1, uint id2);
  void setEdgeBoundaryIndicator(uint id, int value);
  //void fillEdgeBoundaryIndicator(uint id, int value);
  //void fillEdgeSelectedIndicator(uint id, int value);
  //void setEdgeAllBoundaryIndicators(int value);

  
  // Face boundary indicator functions
  uint findFaceFromVertices(uint id1, uint id2, uint id3);
  void setFaceBoundaryIndicator(uint id, int value);
  //void fillFaceBoundaryIndicator(uint id, int value);
  //void fillFaceSelectedIndicator(uint id, int value);
  //void setFaceAllBoundaryIndicators(int value);
  
  virtual void updateMaterial();
  void setVertexSize(int size) { vertex_size_ = size; }
  int  getVertexSize() const   { return vertex_size_; }
  void setEdgeSize(int size)   { edge_size_   = size; }
  int  getEdgeSize() const     { return edge_size_; }
  void drawVertices(bool set);
  bool drawVertices() const    { return draw_vertices_; }
  void drawEdges(bool set);
  bool drawEdges() const       { return draw_edges_; }
  
  void undo();
  void updateUndoBuffers();
  void updateBoundaryIndicatorColors();
  
protected:
  
  bool readVertices();
  bool readIndices();
  bool buildOSGNode();
  void updateTransparency();
  
private:

  // Vertex boundary indicator functions
  //void recursiveVertexFill(uint id, int value);
  //void recursiveVertexFillSelected(uint id, int hitvalue, int value);
  
  // Edge boundary indicator functions
  //void recursiveEdgeFill(uint id, int value);
  //void recursiveEdgeFillSelected(uint id, int hitvalue, int value);
  
  // Face boundary indicator functions
  //void recursiveFaceFill(uint id, int value);
  //void recursiveFaceFillSelected(uint id, int hitvalue, int value);
  

  int  vertex_size_;
  int  edge_size_;
  bool draw_vertices_;
  bool draw_edges_;
  
  osg::ref_ptr<osg::DrawElementsUInt> osg_dolfin_vertices_;
  osg::ref_ptr<osg::DrawElementsUInt> osg_dolfin_edges_;
  osg::ref_ptr<osg::DrawElementsUInt> osg_dolfin_faces_;
  
  osg::ref_ptr<osg::Vec4Array> color_palette_;
  
  osg::ref_ptr<osg::Vec4Array> vertex_colors_;
  osg::ref_ptr<osg::Vec4Array> edge_colors_;
  osg::ref_ptr<osg::Vec4Array> face_colors_;
  
  osg::ref_ptr<osg::Geode> edgeGeode_;
  osg::ref_ptr<osg::Geometry> edge_;
  
  osg::ref_ptr<osg::Geode> vertexGeode_;
  osg::ref_ptr<osg::Geometry> vertex_;

  DolfinObject& dolfin_object;

};
    
#endif // Osg_OSGDOLFINOBJECT_H
