#ifndef Osg_DrawableObject_h_Included             // -*- C++ -*-
#define Osg_DrawableObject_h_Included

#include <util/SimResState.h>

#include <scenegraph/OsgTransformObject.h>

#include <osg/ref_ptr>
#include <osg/Array>
#include <osg/ShapeDrawable>
#include <osg/Switch>

#include <vector>
#include <QObject>

// Forward declarations
namespace osg
{
  class Material;
  class CullFace;
  class Geode;
  class Texture1D;
}

namespace osgFX
{
   class Scribe;
}

class OsgDrawableObjectConnector;

/**
 * Contains SimResData and properties for rendering them.
 *
 *
 * \author Trond vidar Stensby
 * \author Benjamin Kehlet
 * \date 2010-04-13
 */
// Note: Must inherit QObject first in order to work with moc, 
// see http://doc.qt.nokia.com/latest/moc.html#multiple-inheritance-requires-qobject-to-be-first
class OsgDrawableObject : public QObject, public OsgTransformObject
{
  Q_OBJECT
  
 public:
  
  /** Uncommented */
  OsgDrawableObject();
  
  /** Uncommented */
  OsgDrawableObject(const OsgDrawableObject& obj);
  
  /** Uncommented */
  virtual ~OsgDrawableObject();
 
  /** Uncommented */
  virtual void initialize();
  
  /** Apply a state object to this object. */
  virtual void applyState(const SimResState& s);
  
  /** Get this objects default state. */
  SimResState& getDefaultState();
  
  /** Get this objects default state. */
  const SimResState& getDefaultState() const;
  
  /** Get this objects current state. */
  SimResState& getCurrentState();
  
  /** Get this objects current state. */
  const SimResState& getCurrentState() const;
  
  osg::Geode* getGeode();
  const osg::Geode* getGeode() const;
  
  /** Set current state of this object. If null default set will be set.*/
  void setCurrentState(SimResState* state);
  
  /** Uncommented */
  virtual void addChildObject(SimResState* child, const QString& arg=QString::null);
  
  /** Uncommented */
  virtual osg::BoundingBox getBoundingBox() const;
  
  /** Uncommented */
  virtual void sortPrimitives(osg::Vec3& , osg::Vec3& ) {}
  
  /** Uncommented */
  virtual void updateTransformObject();
  
  /** Uncommented */      
  virtual osg::BoundingSphere computeBound() const;
  
 public slots:
  void stateChanged(int state);  


protected:
  /**
   * Finalize should be called from derived classes after that the
   * osg node has been created (i.e. end of overrided initialize).
   */
  virtual void finalize();
  
  virtual void updateTransparency();
  
  virtual void updateMaterial();
  
  virtual void updateColorAndIntensity();
  
  virtual void updateRenderMode();
      
  virtual void updateSortMode();
      
  virtual void updateCullMode();
      
  virtual void updateTwoSidedLighting();
      
  virtual void updateBaseColor();
      
  virtual void updateWireframeColor();

  virtual void updateHighlight();

  /** Access to internal osg structures */
  const bool &isDataOk() const;
  void setDataOk(const bool onoff);

  osg::Vec4 &diffuse();
  osg::Vec4 &ambient();     
  osg::Vec4 &emission();
  osg::Vec4 &specular();
      
  osg::ref_ptr<osg::Geode>     geode_;
  osg::ref_ptr<osg::Vec4Array> colors_;
  osg::ref_ptr<osg::Vec4Array> secondary_colors_;
  osg::ref_ptr<osg::Material>  material_;
  osg::ref_ptr<osg::CullFace>  cull_face_;
  osg::ref_ptr<osgFX::Scribe>  scribe_;
      
private:

  void updateBoundingSphere();  
  void updateBoundingBox();
  void updateAxes();
  void updateGroundPlane();
  

  // Should we use an osg::ref_ptr here as a SimResState
  // might be reused by more viewers?
  osg::ref_ptr<SimResState> default_simres_state_;
  osg::ref_ptr<SimResState> current_simres_state_;
  
  bool data_is_ok_;
  
  int old_render_mode_;
  
  osg::Vec4 diffuse_;
  osg::Vec4 ambient_;
  osg::Vec4 emission_;
  osg::Vec4 specular_;
  
  /** Optional bounding sphere */
  osg::ref_ptr<osg::Geode>         geode_sphere_;
  osg::ref_ptr<osg::ShapeDrawable> bounding_sphere_;
  
  /** Optional bounding box for highlight modues */
  osg::ref_ptr<osg::MatrixTransform> bbox_trans_;
  osg::ref_ptr<osg::Geode>           bbox_geode_;
  osg::ref_ptr<osg::Geometry>        bbox_geom_;
  osg::ref_ptr<osg::Vec4Array>       bbox_colors_;

  /** Optional axes */
  osg::ref_ptr<osg::Switch>          axes;
  osg::ref_ptr<osg::MatrixTransform> axes_transform;
  osg::ref_ptr<osg::Geometry>        axes_geom;
  osg::ref_ptr<osg::Vec4Array>       axes_colors;

  /** Optional ground (xy) plane */
  osg::ref_ptr<osg::Switch>          ground_plane;
  osg::ref_ptr<osg::MatrixTransform> ground_transform;
  osg::ref_ptr<osg::Geometry>        ground_geom;
  osg::ref_ptr<osg::Vec4Array>       ground_colors;
      

  //OsgDrawableObjectConnector* connector;

};
    
#endif // KafOsg_DrawableObject_h_Included
