#include <scenegraph/OsgMeshObject.h>


#include <osg/BlendFunc>
#include <osg/BoundingSphere>
#include <osg/CullFace>
#include <osg/Geode>
#include <osg/Geometry>
#include <osg/Group>
#include <osg/LightModel>
#include <osg/LineStipple>
#include <osg/Material>
#include <osg/Notify>
#include <osg/PolygonMode>
#include <osg/PolygonOffset>
#include <osg/PrimitiveSet>
#include <osg/ShadeModel>
#include <osg/Shape>
#include <osg/ShapeDrawable>
#include <osgFX/Scribe>
#include <osgUtil/Optimizer>

#include <QColor>

#include <algorithm>

//----------------------------------------------------------
OsgMeshObject::OsgMeshObject()
//----------------------------------------------------------
{
}
    
//----------------------------------------------------------
OsgMeshObject::OsgMeshObject(const OsgMeshObject& obj)
  : OsgDrawableObject(obj)
//----------------------------------------------------------
{
}

//----------------------------------------------------------
OsgMeshObject::~OsgMeshObject()
//----------------------------------------------------------
{
}

//----------------------------------------------------------
void OsgMeshObject::appendFaceIds(std::vector<int>& global_ids)
//----------------------------------------------------------
{
  for (unsigned int i = 0; i < face_ids_.size(); i++)
    global_ids.push_back(face_ids_[i]);
}

//----------------------------------------------------------
void OsgMeshObject::appendVertexIds(std::vector<int>& global_ids)
//----------------------------------------------------------
{
  for (unsigned int i = 0; i < vertex_ids_.size(); i++)
    global_ids.push_back(vertex_ids_[i]);
}

//----------------------------------------------------------
void OsgMeshObject::initialize()
//----------------------------------------------------------
{ 
  OsgDrawableObject::initialize();   
}


//----------------------------------------------------------
void OsgMeshObject::finalize()
//----------------------------------------------------------
{
  OsgDrawableObject::finalize();
}

//----------------------------------------------------------
void OsgMeshObject::sortPrimitives(osg::Vec3& eye, osg::Vec3& center)
//----------------------------------------------------------
{
  // Check if this node can be Z sorted.
  if(sorted_indices_.size()==0) return;

  osg::DrawElementsUInt& o_indx = *orig_indices_.get();
  osg::DrawElementsUInt& c_indx = *curr_indices_.get();
  osg::Vec3Array& vert = *vertices_.get();
	
  unsigned int i;

  osg::Vec3 view = center-eye;

  std::vector<ZSortElem>::iterator it;
  for(i=0, it=sorted_indices_.begin() ; it != sorted_indices_.end() ; it++, i++)
  {	
    double dist;
    //int j;

    int ind = 3*it->second;
    const osg::Vec3& v0 = vert[o_indx[ind]];
    const osg::Vec3& v1 = vert[o_indx[ind+1]];
    const osg::Vec3& v2 = vert[o_indx[ind+2]];

    osg::Vec3 v = (v0+v1+v2)/3.0;
    dist = (v-eye)*view;
    // dist = (v-eye).length();
    
    it->first = -dist;
  }

  sort(sorted_indices_.begin(), sorted_indices_.end());
  
  for(i=0, it=sorted_indices_.begin() ; it != sorted_indices_.end() ; it++, i++)
  {
    c_indx[3*i+0] = o_indx[3*it->second+0];
    c_indx[3*i+1] = o_indx[3*it->second+1];
    c_indx[3*i+2] = o_indx[3*it->second+2];
  }

  mesh_->setPrimitiveSet(0, curr_indices_.get());    
}

//----------------------------------------------------------
void OsgMeshObject::appendVertices(osg::Matrix& mat, osg::Vec3Array& vertices)
//----------------------------------------------------------
{
  //   printf("OsgMeshObject::writeTetmeshPoints, label=%s...\n", getLabel().latin1());

  for (unsigned int i=0; i<(*vertices_).size(); i++)
  {
    osg::Vec3 res = mat.preMult((*vertices_)[i]);

    //      vertices.push_back(osg::Vec4(res,boundary_id_));
    vertices.push_back(res);

    //printf("orig = %lf %lf %lf, res = %lf %lf %lf...\n", org.x(), org.y(), org.z(), res.x(), res.y(), res.z());

    //fprintf(pointfile, "%lf %lf %lf %d\n", res.x(), res.y(), res.z(), 1);
  }
}

//----------------------------------------------------------
void OsgMeshObject::appendFaces(osg::DrawElementsUInt& faces, int offset)
//----------------------------------------------------------
{
  //   printf("OsgMeshObject::writeTetmeshFaces...\n");

  for (unsigned int i = 0; i < (*curr_indices_).size(); i++)
    faces.push_back((*curr_indices_)[i]+offset+1);

  /*
    fprintf(facesfile, "3 %d %d %d 1 1 1 1\n", 
    (int) curr_indices[3*i]  +offset+1, 
    (int) curr_indices[3*i+1]+offset+1, 
    (int) curr_indices[3*i+2]+offset+1);
  */
}

//----------------------------------------------------------
int OsgMeshObject::getNumFaces() 
//----------------------------------------------------------
{ 
  return curr_indices_->size()/3; 
}

//----------------------------------------------------------
void OsgMeshObject::updateBaseColor()
//----------------------------------------------------------
{
  OsgDrawableObject::updateBaseColor();

  if(mesh_.get()) 
  {
    osg::Vec4Array* colors = dynamic_cast<osg::Vec4Array*>(mesh_->getColorArray());
    if(colors) 
    {
      osg::Vec4 color, scolor;
      const SimResState &current_state = getCurrentState();
	  
      if (0) { //current_state.getHighlight()) {
        color.x() = current_state.getHighlightColor().red()  /255.0;
        color.y() = current_state.getHighlightColor().green()/255.0;
        color.z() = current_state.getHighlightColor().blue() /255.0;
        color.w() = 1.0 - current_state.getTransparency();
        scolor = color;
      }
      else 
      {
        color.x() = current_state.getBaseColor().red()  /255.0;
        color.y() = current_state.getBaseColor().green()/255.0;
        color.z() = current_state.getBaseColor().blue() /255.0;
        color.w() = 1.0 - current_state.getTransparency();

        scolor.x() = current_state.getBackColor().red()  /255.0;
        scolor.y() = current_state.getBackColor().green()/255.0;
        scolor.z() = current_state.getBackColor().blue() /255.0;
        scolor.w() = 1.0 - current_state.getTransparency();
      }
	  
      osg::Vec4Array::iterator it;
      for(it=colors->begin(); it!=colors->end(); it++) 
      {
        (*it) = color;
      }
      mesh_->setColorArray(colors);
      
      if (secondary_colors_.valid()) 
      {
        for(it=secondary_colors_->begin(); it!=secondary_colors_->end(); it++) 
        {
          *it = scolor;
        }
        mesh_->setSecondaryColorArray(secondary_colors_.get());
      }
    }
  }
}

//----------------------------------------------------------
void OsgMeshObject::updateTransparency()
//----------------------------------------------------------
{
  //printf("OsgMeshObject::updateTransparency\n");
  OsgDrawableObject::updateTransparency();

  osg::Vec4Array* vec = colors_.get();

  if(vec && mesh_.get()) 
  {
    mesh_->setColorArray(vec);
    // mesh_->setColorBinding(osg::Geometry::BIND_OVERALL);
  }

  if(secondary_colors_.get() && mesh_.get()) 
  {
    mesh_->setSecondaryColorArray(secondary_colors_.get());
  }
}
//----------------------------------------------------------
void OsgMeshObject::updateTransformObject()
//----------------------------------------------------------
{
  OsgDrawableObject::updateTransformObject();
}
