#ifndef Osg_MeshObject_h_Included             // -*- C++ -*-
#define Osg_MeshObject_h_Included

#include <scenegraph/OsgDrawableObject.h>
#include <osg/Array>
#include <vector>

// Forward declarations
namespace osg
{
   class DrawElementsUInt;
   class Geometry;
   class Texture1D;
}

namespace osgFX
{
   class Scribe;
}

/**
 * Contains SimResData and properties for rendering them.
 *
 * \ingroup scenegraph
 *
 * @author Trond vidar Stensby
 */

class OsgMeshObject : public OsgDrawableObject
{
  
public:

  /** Uncommented */
  OsgMeshObject();
  
  /** Uncommented */
  OsgMeshObject(const OsgMeshObject& obj);
  
  /** Uncommented */
  virtual ~OsgMeshObject();
  
  /** Uncommented */
  virtual void initialize();
  
  /** Uncommented */
  virtual void sortPrimitives(osg::Vec3& eye, osg::Vec3& center);
  
  /** Uncommented */
  void appendVertices(osg::Matrix& mat, osg::Vec3Array& vertices);
  
  /** Uncommented */
  void appendFaces(osg::DrawElementsUInt& faces, int offset);
  
  /** Uncommented */
  int getNumVertices() { return vertices_->size(); }

  /** Uncommented */
  osg::Vec3Array* getVertexArray() { return vertices_.get(); }
  
  /** Uncommented */
  osg::Vec3 getVertex(int i) { return (*vertices_)[i]; }

  /** Uncommented */
  int getNumFaces();
  
  /** Uncommented */
  void appendFaceIds(std::vector<int>& global_ids);
  
  /** Uncommented */
  void appendVertexIds(std::vector<int>& global_ids);
  
  /** Uncommented */
  virtual void updateTransparency();
  
  /** Uncommented */
  virtual void updateBaseColor();
  
  /** Uncommented */
  //MeshObject* getMeshObject();
  
  /** Uncommented */
  virtual void updateTransformObject();
  
protected:
  /**
   * Finalize should be called from derived classes after that the
   * osg node has been created (i.e. end of overrided initialize).
   */
  virtual void finalize();
  
  /** Holds the face ids of the object.*/
  std::vector<int> face_ids_;
  
  /** Holds the vertex ids of the object.*/
  std::vector<int> vertex_ids_;
  
  osg::ref_ptr<osg::Vec3Array> vertices_;
  osg::ref_ptr<osg::Vec3Array> normals_;
  osg::ref_ptr<osg::Vec2Array> texcoords_;
  
  osg::ref_ptr<osg::DrawElementsUInt> orig_indices_;
  osg::ref_ptr<osg::DrawElementsUInt> curr_indices_;
  
  osg::ref_ptr<osg::Geometry> mesh_;
  
  typedef std::pair<double,unsigned int> ZSortElem;
  std::vector<ZSortElem> sorted_indices_; 
  
};

#endif // Osg_MeshObject_h_Included
