#ifndef Osg_TransformObject_h_Included             // -*- C++ -*-
#define Osg_TransformObject_h_Included

#include <base/ObjectState.h>

#include <osg/MatrixTransform>
#include <osg/Vec3>
#include <osg/BoundingBox>
#include <util/SimResState.h>

/**
 * 
 *
 * Provides transformations to the scenegraph object
 *
 * \author Christian Tarrou
 * \author Benjamin Kehlet
 * \date 2010-03-29
 */

class OsgTransformObject : public osg::MatrixTransform
{

public:
  
  /** Uncommented */
  OsgTransformObject();
  
  /** Uncommented */
  OsgTransformObject(const OsgTransformObject& obj);
  
  /** Uncommented */
  virtual ~OsgTransformObject();
  
  /** Uncommented */
  virtual void setName(QString s);
  
  /** Uncommented */
  virtual void initialize();
  
  /** Uncommented */
  const osg::Vec3 &getTranslation() const;
  
  /** Uncommented */
  void setTranslation(const osg::Vec3 &translation);
  
  /** Uncommented */
  const osg::Vec3 &getRotationAxis() const;
  
  /** Uncommented */
  void setRotationAxis(const osg::Vec3 &rot_axis);
  
  /** Uncommented */
  const double &getRotationAngle() const;

  /** Uncommented */
  void setRotationAngle(double rot_angle);

  /** Uncommented */
  const osg::Vec3 &getScale() const;

  /** Uncommented */
  void setScale(const osg::Vec3 &scale);

  /** Set scale orientation component. */
  void setScaleOrientation(const osg::Quat &so);
    
  /** Set scale orientation component. */
  const osg::Quat &getScaleOrientation() const;

  void setVisible(bool visible);
  
  bool getVisible() const;

  /** Uncommented */
  double getEulerXAngle() const { return euler_x_angle_; }

  /** Uncommented */
  void setEulerXAngle(double euler_x_angle);

  /** Uncommented */
  double getEulerYAngle() const { return euler_y_angle_; }

  /** Uncommented */
  void setEulerYAngle(double euler_y_angle);

  /** Uncommented */
  double getEulerZAngle() const { return euler_z_angle_; }

  /** Uncommented */
  void setEulerZAngle(double euler_z_angle);

  /** Uncommented */
  virtual osg::BoundingBox getBoundingBox() const = 0;

  /** Uncommented */
  osg::Matrix getCombinedTransform();

  /** Uncommented */
  osg::Matrix getCombinedTransform(osg::Matrix mat_in);

  /** Uncommented */
  void setObjectManipMode(bool object_manip_mode) { object_manipulation_mode_ = object_manip_mode; }

  /** Uncommented */
  void updateTransRotScale();

  /** Uncommented */
  virtual void updateBoundingBox(osg::BoundingBox) {}

  /** Uncommented */
  virtual void updateTransformObject();

  /** Returns true if topology has changed */
  bool hasTopologyChanged() const;

  /** Clears topology state (changes done) */
  void clearTopologyState();

  /** Returns true if geometry has changed */
  bool hasGeometryChanged() const;

  /** Clears geometry state (changes done) */
  void clearGeometryState();

  ObjectState &drawingState();

  /** Returns true if drawing state has changed */
  bool hasDrawingChanged() const;

  /** Clears drawing state (changes done) */
  void clearDrawingState();

  /** Uncommented */
  static void getAxisAngle(double& angle, osg::Vec3& axis, osg::Matrix mat, osg::Vec3 scale);

  /** Uncommented */
  static osg::Vec3 getTranslation(osg::Matrix mat);

  /** Uncommented */
  void updateMatrix();

protected:
  /**
   * Finalize should be called from derived classes after that the
   * osg node has been created (i.e. end of overrided initialize).
   */
  virtual void finalize();

  void updateEulerAngles(osg::Quat q);

  /** Uncommented */
  double euler_x_angle_;

  /** Uncommented */
  double euler_y_angle_;

  /** Uncommented */
  double euler_z_angle_;

  /** Uncommented */
  bool euler_rotation_mode_;

  /** Uncommented */
  bool object_manipulation_mode_;

  /** Topology changed state */
  ObjectState topo_state_orig;
  ObjectState topo_state_;

  /** Geometry changed state */
  ObjectState geom_state_orig;
  ObjectState geom_state_;

  /** Drawing changed state */
  ObjectState draw_state_orig;
  ObjectState draw_state_;

 private :
  osg::Vec3 rot_axis_;  
  osg::Vec3 translation_;
  osg::Vec3 scale_;
  osg::Quat so_;
  double rot_angle_;
  bool visible_;

};

#endif // Osg_TransformObject_h_Included
