#include <util/QtUtil.h>

#include <QString>
#include <QFile>
#include <QFileInfo>
#include <QDir>


static int _table_row_ = 0;

//----------------------------------------------------------
void QtUtil::recursiveBlockSignals(QObject* obj, bool b)
//----------------------------------------------------------
{
  if (obj) {
    obj->blockSignals(b);
    const QObjectList& list = obj->children();
    
    QObjectList::const_iterator it;
    for(it=list.begin() ; it != list.end() ; it++) {
      QObject* o = *it;
      if (o != obj) recursiveBlockSignals(o, b);
    }
  }
}

//----------------------------------------------------------
osg::Vec4 QtUtil::qtToOsg(const QColor qcol)
//----------------------------------------------------------
{
  osg::Vec4 res;
  res.set(qcol.redF(), qcol.greenF(), qcol.blueF(), qcol.alphaF());
  return res;
}

//----------------------------------------------------------
QColor QtUtil::osgToQt(const osg::Vec4 col)
//----------------------------------------------------------
{
  int r = (int)(col[0]*255.0);
  int g = (int)(col[1]*255.0);
  int b = (int)(col[2]*255.0);
  int a = (int)(col[3]*255.0);
  
  return QColor(r,g,b,a);      
}

//----------------------------------------------------------
QString QtUtil::toStr(osg::Vec3 v)
//----------------------------------------------------------
{
  return QString("%1 %2 %3").
    arg(QString::number(v.x())).
    arg(QString::number(v.y())).
    arg(QString::number(v.z()));
}

//----------------------------------------------------------
QString QtUtil::toStr(osg::Vec4 v)
//----------------------------------------------------------
{
  return QString("%1 %2 %3 %4").
    arg(QString::number(v.x())).
    arg(QString::number(v.y())).
    arg(QString::number(v.z())).
    arg(QString::number(v.w()));
}

//----------------------------------------------------------
QString QtUtil::toStr(osg::Vec3 v, double w)
//----------------------------------------------------------
{
  return QString("%1 %2 %3 %4").
    arg(QString::number(v.x())).
    arg(QString::number(v.y())).
    arg(QString::number(v.z())).
    arg(QString::number(w));
}

//----------------------------------------------------------
QString QtUtil::toStr(QString txt)
//----------------------------------------------------------
{
  txt.replace("<", "&lt;");
  txt.replace(">", "&gt;");
  return txt;
}

//----------------------------------------------------------
void QtUtil::beginTable(QTextStream &os, const QString &entry, const QString &description)
//----------------------------------------------------------
{
  _table_row_ = 0;
  os << "<table cellpadding=<\"4\" cellspacing=\"2\" border=\"0\" width=\"80%\"> \n";
  os << "<tr bgcolor=\"#a2c511\"> <th valign=\"top\">" << entry << "<th valign=\"top\">"
     << description << " \n";
}

//----------------------------------------------------------
void QtUtil::newRow(QTextStream &os, const QString &entry, const QString &description)
//----------------------------------------------------------
{
  if (_table_row_%2)
    os << "<tr bgcolor=\"#f0f0f0\">";
  else
    os << "<tr bgcolor=\"#d0d0d0\">";
  os << "<td valign=\"top\">" << entry << "\n";
  os << "<td valign=\"top\">" << description << "\n";
  _table_row_++;
}

//----------------------------------------------------------
void QtUtil::endTable(QTextStream &os)
//----------------------------------------------------------
{
  os << "</table></center>\n";
}

//----------------------------------------------------------
QStringList QtUtil::findNewFiles(const QString &dir_path, const QDateTime &date)
//----------------------------------------------------------
{
  QStringList res;
  QDir        dir(dir_path);
  
  QFileInfoList files = dir.entryInfoList(QDir::Files | QDir::Hidden);
  QFileInfoList::Iterator it = files.begin();
  for (; it != files.end(); ++it) {
    QFileInfo &fi = *it;
    if (date.secsTo(fi.lastModified()) >= 0) {
      res << fi.fileName();
    }
  }
  
  return res;
}

