#include <util/SimResState.h>
#include <QDebug>

//----------------------------------------------------------
SimResState::SimResState()
  : render_mode_(SMOOTH),
    sort_mode_(0),
    culling_(0),
    highlight_(false),
    two_sided_(true),
    transparency_(0.0),
    diffuse_intensity_(0.56),
    ambient_intensity_(0.4),
    emission_intensity_(0.04),
    specular_intensity_(0.0),
    shininess_(0.0),
    base_color_(0xA0, 0xA0, 0xA0),
    back_color_(0x00, 0x00, 0x00),
    highlight_color_(255, 255, 0),
    wireframe_color_(0, 0, 0),
    point_size_(1.0),
    line_width_(1.0),
    contour_(false),
    contour_nb_(20),
    show_bound_(false),
    show_triangle_normal_(false),
    show_axes_(false)
//----------------------------------------------------------
{
  setObjectName("SimResState");
}
  
//----------------------------------------------------------
SimResState::SimResState(const SimResState& obj)
  : osg::Referenced(obj),
    render_mode_(obj.render_mode_),
    sort_mode_(obj.sort_mode_),
    culling_(obj.culling_),
    highlight_(obj.highlight_),
    two_sided_(obj.two_sided_),
    transparency_(obj.transparency_),
    diffuse_intensity_(obj.diffuse_intensity_),
    ambient_intensity_(obj.ambient_intensity_),
    emission_intensity_(obj.emission_intensity_),
    specular_intensity_(obj.specular_intensity_),
    shininess_(obj.shininess_),
    base_color_(obj.base_color_),
    back_color_(obj.back_color_),
    highlight_color_(obj.highlight_color_),
    wireframe_color_(obj.wireframe_color_),
    point_size_(obj.point_size_),
    line_width_(obj.line_width_),
    contour_(obj.contour_),
    contour_nb_(obj.contour_nb_),
    show_bound_(obj.show_bound_),
    show_triangle_normal_(obj.show_triangle_normal_),
    show_axes_(obj.show_axes_)
    
//----------------------------------------------------------
{
}  
//----------------------------------------------------------
SimResState::~SimResState()
//----------------------------------------------------------
{
}
//----------------------------------------------------------
bool SimResState::setTransparency(double transparency)
//----------------------------------------------------------
{
  if(transparency_ != transparency) {
    transparency_ = transparency;
    emit stateChanged(Transparency);
    return true;
  }
  else 
    return false;
}

//----------------------------------------------------------
const double &SimResState::getTransparency() const 
//----------------------------------------------------------
{
  return transparency_;
}
    
//----------------------------------------------------------
bool SimResState::setDiffuseIntensity(double value)
//----------------------------------------------------------
{
  if(diffuse_intensity_ != value) {
    diffuse_intensity_ = value;
    emit stateChanged(DiffuseIntensity);
    return true;
  }
  else 
    return false;
}

//----------------------------------------------------------
const double &SimResState::getDiffuseIntensity() const 
//----------------------------------------------------------
{
  return diffuse_intensity_;
}

//----------------------------------------------------------
bool SimResState::setAmbientIntensity(double value)
//----------------------------------------------------------
{
  if(ambient_intensity_ != value) {
    ambient_intensity_ = value;
    emit stateChanged(AmbientIntensity);
    return true;
  }
  else 
    return false;
}

//----------------------------------------------------------
const double &SimResState::getAmbientIntensity() const 
//----------------------------------------------------------
{
  return ambient_intensity_;
}

//----------------------------------------------------------
bool SimResState::setEmissionIntensity(double value)
//----------------------------------------------------------
{
  if(emission_intensity_ != value) {
    emission_intensity_ = value;
    emit stateChanged(EmissionIntensity);
    return true;
  }
  else 
    return false;
}

//----------------------------------------------------------
const double &SimResState::getEmissionIntensity() const 
//----------------------------------------------------------
{
  return emission_intensity_;
}

//----------------------------------------------------------
bool SimResState::setSpecularIntensity(double value)
//----------------------------------------------------------
{
  if(specular_intensity_ != value) {
    specular_intensity_ = value;
    emit stateChanged(SpecularIntensity);
    return true;
  }
  else 
    return false;
}

//----------------------------------------------------------
const double &SimResState::getSpecularIntensity() const 
//----------------------------------------------------------
{
  return specular_intensity_;
}

//----------------------------------------------------------
bool SimResState::setShininess(double value)
//----------------------------------------------------------
{
  if (shininess_ != value) {
    shininess_ = value;
    emit stateChanged(Shininess);
    return true;
  }
  else 
    return false;
}
  
//----------------------------------------------------------
const double &SimResState::getShininess() const 
//----------------------------------------------------------
{
  return shininess_;
}

//----------------------------------------------------------
bool SimResState::setRenderMode(int mode)
//----------------------------------------------------------
{
  if(render_mode_ != mode) {
    //printf("render_mode_ != mode...\n");
    render_mode_ = mode;
    emit stateChanged(RenderMode);
    return true;
  }
  else 
    return false;
}
  
//----------------------------------------------------------
int SimResState::getRenderMode() const 
//----------------------------------------------------------
{
  return render_mode_;
}

//----------------------------------------------------------
bool SimResState::setSortMode(int mode)
//----------------------------------------------------------
{
  if(sort_mode_ != mode) {
    sort_mode_ = mode;
    emit stateChanged(SortMode);
    return true;
  }
  else 
    return false;
}
  
//----------------------------------------------------------
const int &SimResState::getSortMode() const 
//----------------------------------------------------------
{
  return sort_mode_;
}

//----------------------------------------------------------
int SimResState::setCullMode(int mode)
//----------------------------------------------------------
{
  if(culling_ != mode) {
    culling_ = mode;
    emit stateChanged(CullMode);
    return true;
  }
  else 
    return false;
}
  
//----------------------------------------------------------
const int &SimResState::getCullMode() const 
//----------------------------------------------------------
{
  return culling_;
}

//----------------------------------------------------------
bool SimResState::setHighlight(bool mode)
//----------------------------------------------------------
{
  if(highlight_ != mode) {
    highlight_ = mode;
    emit stateChanged(Highlight);
    return true;
  }
  else 
    return false;
}
    
//----------------------------------------------------------
const bool &SimResState::getHighlight() const 
//----------------------------------------------------------
{
  return highlight_;
}

//----------------------------------------------------------
bool SimResState::setShowAxes(bool mode)
//----------------------------------------------------------
{
  if(show_axes_ != mode) {
    show_axes_ = mode;
    emit stateChanged(Axes);
    return true;
  }
  else 
    return false;
}    
//----------------------------------------------------------
bool SimResState::getShowAxes() const 
//----------------------------------------------------------
{
  return show_axes_;
}

//----------------------------------------------------------
bool SimResState::setShowPlane(bool mode)
//----------------------------------------------------------
{
  if(show_plane_ != mode) {
    show_plane_ = mode;
    emit stateChanged(Plane);
    return true;
  }
  else 
    return false;
}
//----------------------------------------------------------
bool SimResState::getShowPlane() const 
//----------------------------------------------------------
{
  return show_plane_;
}

//----------------------------------------------------------
bool SimResState::setTwoSidedLighting(bool mode)
//----------------------------------------------------------
{
  if(two_sided_ != mode) {
    two_sided_ = mode;
    emit stateChanged(TwoSidedLighting);
    return true;
  }
  else 
    return false;
}
    
//----------------------------------------------------------
const bool &SimResState::getTwoSidedLighting() const 
//----------------------------------------------------------
{
  return two_sided_;
}

//----------------------------------------------------------
bool SimResState::setBaseColor(QColor color)
//----------------------------------------------------------
{
  if(base_color_ != color) {
    base_color_ = color;
    emit stateChanged(BaseColor);
    return true;
  }
  else 
    return false;
}
  
//----------------------------------------------------------
const QColor &SimResState::getBaseColor() const 
//----------------------------------------------------------
{
  return base_color_;
}

//----------------------------------------------------------
bool SimResState::setBackColor(QColor color)
//----------------------------------------------------------
{
  if(back_color_ != color) {
    back_color_ = color;
    emit stateChanged(BackColor);
    return true;
  }
  else 
    return false;
}
  
//----------------------------------------------------------
const QColor &SimResState::getBackColor() const 
//----------------------------------------------------------
{
  return back_color_;
}

//----------------------------------------------------------
bool SimResState::setHighlightColor(QColor color)
//----------------------------------------------------------
{
  if(highlight_color_ != color) {
    highlight_color_ = color;
    emit stateChanged(HighlightColor);
    return true;
  }
  else 
    return false;
}
  
//----------------------------------------------------------
const QColor& SimResState::getHighlightColor() const 
//----------------------------------------------------------
{
  return highlight_color_;
}

//----------------------------------------------------------
bool SimResState::setWireframeColor(QColor color)
//----------------------------------------------------------
{
  if(wireframe_color_ != color) {
    wireframe_color_ = color;
    emit stateChanged(WireframeColor);
    return true;
  }
  else 
    return false;
} 
//----------------------------------------------------------
const QColor &SimResState::getWireframeColor() const 
//----------------------------------------------------------
{
  return wireframe_color_;
}

//----------------------------------------------------------
bool SimResState::setPointSize(double point_size) 
//----------------------------------------------------------
{
  if (point_size_ != point_size) {
    point_size_ = point_size; 
    emit stateChanged(PointSize);
    return true;
  }
  else 
    return false;
}

//----------------------------------------------------------
const double &SimResState::getPointSize() const 
//----------------------------------------------------------
{
  return point_size_; 
}

//----------------------------------------------------------
bool SimResState::setLineWidth(double line_width) 
//----------------------------------------------------------
{
  if (line_width_ != line_width) {
    line_width_ = line_width; 
    emit stateChanged(LineWidth);
    return true;
  }
  else 
    return false;
}

//----------------------------------------------------------
const double &SimResState::getLineWidth() const 
//----------------------------------------------------------
{
  return line_width_; 
}

//----------------------------------------------------------
bool SimResState::setContour(const bool onoff) 
//----------------------------------------------------------
{
  if (contour_ != onoff) {
    contour_ = onoff; 
    emit stateChanged(Contour);
    return true;
  }
  else 
    return false;
}

//----------------------------------------------------------
const bool &SimResState::getContour() const 
//----------------------------------------------------------
{
  return contour_; 
}

//----------------------------------------------------------
bool SimResState::setContourNb(const int nb)
//----------------------------------------------------------
{
  if (contour_nb_ != nb) {
    contour_nb_ = nb;
    emit stateChanged(ContourNb);
    return true;
  }
  else 
    return false;
}
  
//----------------------------------------------------------
const int &SimResState::getContourNb() const 
//----------------------------------------------------------
{
  return contour_nb_; 
}

//----------------------------------------------------------
bool SimResState::showBoundingSphere(const bool onoff)
//----------------------------------------------------------
{
  if (show_bound_ != onoff) {
    show_bound_ = onoff;
    emit stateChanged(BoundingSphere);
    return true;
  }
  else 
    return false;
}
  
//----------------------------------------------------------
const bool &SimResState::isBoundingSphereShown() const 
//----------------------------------------------------------
{
  return show_bound_; 
}

//----------------------------------------------------------
bool SimResState::showTriangleNormals(const bool onoff)
//----------------------------------------------------------
{
  if (show_triangle_normal_ != onoff) {
    show_triangle_normal_ = onoff;
    emit stateChanged(TriangleNormals);
    return true;
  }
  else 
    return false;
}

//----------------------------------------------------------
const bool &SimResState::areTriangleNormalsShown() const
//----------------------------------------------------------
{
  return show_triangle_normal_; 
}
